import sys
import os
import time
import platform

def getGWAMARPathRun(run_path, cmd_path):
    full_path = os.path.abspath(os.path.join(run_path, cmd_path))
    gwamar_path = os.path.dirname(full_path)
    return gwamar_path + "/"

run_path = os.path.abspath(os.curdir)
cmd_path = sys.argv[0]
gwamar_path = getGWAMARPathRun(run_path, cmd_path)
os.chdir(gwamar_path)

sys.path += ["src/"]

from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_utils import logExecutionTime

if sys.version.startswith("3"):
    raw_input = input

def createCommandParams(attrs, parameters):
    new_cmd = ""
    for key in attrs:
        if key in parameters:
            new_cmd += " " + key + "=" + parameters[key]
    return new_cmd

def prepare(parameters):
    os.chdir(os.path.abspath("src/drsoft/prepare/"))

    params = ["D", "REVORDER", "SCORES", "OW", "W", "GEN_BIN_PROFILES", 
     "MT", "EP", "TREE_ID"]
    passed_params = createCommandParams(params, parameters)

    t0_time = time.time()
    if os.system(python_cmd + " p0_save_strains_ordered.py" + passed_params): exit()
    if os.system(python_cmd + " p1_prepare_experiments.py" + passed_params): exit()
    if os.system(python_cmd + " p2_convert_profiles_genes.py" + passed_params): exit()
    if os.system(python_cmd + " p3_convert_profiles_muts.py" + passed_params): exit()
    if os.system(python_cmd + " p4_gen_binary_trees.py" + passed_params): exit()
    if os.system(python_cmd + " p5_cmp_resistance_profiles.py" + passed_params): exit()
    t1_time = time.time()
    os.chdir("../../../")
    logExecutionTime(t1_time - t0_time, "prepare")
    
def scores_std(parameters):
    os.chdir(os.path.abspath("src/drsoft/scoring/"))

    params = ["D", "REVORDER", "SCORES", "OW", "W", "GEN_BIN_PROFILES", 
    "MT", "EP"]
    passed_params = createCommandParams(params, parameters)

    print(passed_params)
    t0_time = time.time()
    if os.system(python_cmd + " sc1_compute_std_scores.py" + passed_params): exit()
    if os.system(python_cmd + " sc2_save_std_rankings.py" + passed_params): exit()
    t1_time = time.time()
    os.chdir("../../../")
    logExecutionTime(t1_time - t0_time, "scores_std")

def scores_ranks(parameters):
    os.chdir(os.path.abspath("src/drsoft/scoring/"))

    params = ["D", "REVORDER", "SCORES", "OW", "W", "GEN_BIN_PROFILES", 
     "MT", "EP"]
    passed_params = createCommandParams(params, parameters)

    print(passed_params)
    t0_time = time.time()
    if os.system(python_cmd + " sc2_save_std_rankings.py" + passed_params): exit()
    t1_time = time.time()
    os.chdir("../../../")
    logExecutionTime(t1_time - t0_time, "scores_ranks")
    
def scores_pvalues(parameters):
    os.chdir(os.path.abspath("src/drsoft/scoring/"))

    params = ["D", "REVORDER", "SCORES",  "OW", "W", "GEN_BIN_PROFILES", 
     "MT", "EP"]
    passed_params = createCommandParams(params, parameters)

    t0_time = time.time()
    if os.system(python_cmd + " sc3_compute_perm_scores.py" + passed_params): exit()
    if os.system(python_cmd + " sc4_save_perm_rankings.py" + passed_params): exit()
    t1_time = time.time()
    os.chdir("../../../")
    logExecutionTime(t1_time - t0_time, "scores_pvalues")
    
def scores_norm(parameters):
    os.chdir(os.path.abspath("src/drsoft/scoring/"))

    params = ["D", "REVORDER", "SCORES",  "OW", "W", "GEN_BIN_PROFILES", 
     "MT", "EP"]
    passed_params = createCommandParams(params, parameters)

    t0_time = time.time()
    if os.system(python_cmd + " sc7_compute_norm_scores.py" + passed_params): exit()
    t1_time = time.time()
    os.chdir("../../../")
    logExecutionTime(t1_time - t0_time, "scores_norm")
    

def perm_test(parameters):
    os.chdir(os.path.abspath("src/drsoft/permtest/"))

    params = ["D", "REVORDER", "SCORES",  "OW", "W", "GEN_BIN_PROFILES", 
     "MT", "EP"]
    passed_params = createCommandParams(params, parameters)

    t0_time = time.time()
    if os.system(python_cmd + " perm1_generate_res_profiles.py" + passed_params): exit()
    if os.system(python_cmd + " perm2_compute_perm_rep_scores.py" + passed_params): exit()
    if os.system(python_cmd + " perm3_save_perm_rep_rankings.py" + passed_params): exit()
    if os.system(python_cmd + " perm4_compute_perm_comb_scores.py" + passed_params): exit()
    if os.system(python_cmd + " perm5_combine_perm_rep_scores.py" + passed_params): exit()
    if os.system(python_cmd + " perm6_calc_pvalues.py" + passed_params): exit()
    t1_time = time.time()
    os.chdir("../../../")
    logExecutionTime(t1_time - t0_time, "perm_test")
    
def analysis1(parameters):
    os.chdir(os.path.abspath("src/drsoft/analysis/"))

    params = ["D", "REVORDER", "SCORES", "OW", "W", "GEN_BIN_PROFILES", 
     "MT", "EP", "SCORE_SORT"]
    passed_params = createCommandParams(params, parameters)

    t0_time = time.time()
    if os.system(python_cmd + " a1_save_details_scores_all.py" + passed_params): exit()
    if os.system(python_cmd + " a2_save_details_scores_sel.py" + passed_params): exit()
 
    t1_time = time.time()
    os.chdir("../../../")
    logExecutionTime(t1_time - t0_time, "analysis")

def analysis2(parameters):
    os.chdir(os.path.abspath("src/drsoft/analysis/"))

    params = ["D", "REVORDER", "SCORES", "OW", "W", "GEN_BIN_PROFILES", 
     "MT", "EP", "SCORE_SORT"]
    passed_params = createCommandParams(params, parameters)

    t0_time = time.time()
    if os.system(python_cmd + " a3_save_associations_all.py" + passed_params): exit()
    if os.system(python_cmd + " a4_save_associations_sel.py" + passed_params): exit()
    
    t1_time = time.time()
    os.chdir("../../../")
    logExecutionTime(t1_time - t0_time, "analysis2")

def other(parameters):
    os.chdir(os.path.abspath("src/drsoft/other/"))

    params = ["D", "REVORDER", "SCORES", "OW", "W", "GEN_BIN_PROFILES", 
     "MT", "EP"]
    passed_params = createCommandParams(params, parameters)

    t0_time = time.time()
    if os.system(python_cmd + " show0_gold_associations.py" + passed_params): exit()
    
    t1_time = time.time()
    os.chdir("../../../")
    logExecutionTime(t1_time - t0_time, "other")

def comparisonF(parameters):
    os.chdir(os.path.abspath("src/drsoft/comparison/"))

    params = ["D", "REVORDER", "SCORES", "OW", "W", "GEN_BIN_PROFILES", 
    "MT",  "EP", "RAND", "SUBSETS", "STATS"]
    passed_params = createCommandParams(params, parameters)

    t0_time = time.time()
    #if os.system(python_cmd + " cmp2_auc_stats.py" + passed_params): exit()
    if os.system(python_cmd + " cmp3_draw_figures.py" + passed_params): exit()
    t1_time = time.time()
    os.chdir("../../../")
    logExecutionTime(t1_time - t0_time, "comparisonF")

def comparison3(parameters):
    os.chdir(os.path.abspath("src/drsoft/comparison/"))

    params = ["D", "REVORDER", "SCORES", "OW", "W", "GEN_BIN_PROFILES", 
     "MT",  "EP", "RAND", "SUBSETS", "STATS"]
    passed_params = createCommandParams(params, parameters)

    t0_time = time.time()
    if os.system(python_cmd + " cmp2_auc_stats.py" + passed_params): exit()
    t1_time = time.time()
    os.chdir("../../../")
    logExecutionTime(t1_time - t0_time, "comparison2")

def comparison2(parameters):
    os.chdir(os.path.abspath("src/drsoft/comparison/"))

    params = ["D", "REVORDER", "SCORES", "OW", "W", "GEN_BIN_PROFILES", 
     "MT",  "EP", "RAND", "SUBSETS", "STATS"]
    passed_params = createCommandParams(params, parameters)

    t0_time = time.time()
    if os.system(python_cmd + " cmp1_prec-recall_stats.py" + passed_params): exit()
    t1_time = time.time()
    os.chdir("../../../")
    logExecutionTime(t1_time - t0_time, "comparison2")

def comparison1(parameters):
    os.chdir(os.path.abspath("src/drsoft/comparison/"))

    params = ["D", "REVORDER", "SCORES", "OW", "W", "GEN_BIN_PROFILES", 
     "MT", "EP", "SUBSETS"]
    passed_params = createCommandParams(params, parameters)

    t0_time = time.time()
    if os.system(python_cmd + " cmp0_save_associations.py" + passed_params): exit()
    t1_time = time.time()
    os.chdir("../../../")
    logExecutionTime(t1_time - t0_time, "comparison1")

def comp_mutations(parameters):
    os.chdir(os.path.abspath("src/drsoft/compensatory/"))

    params = ["D", "REVORDER", "SCORES", "OW", "W", "GWAMAR_DATASETS"]
    passed_params = createCommandParams(params, parameters)

    t0_time = time.time()
    if os.system(python_cmd + " show0_save_putative_muts.py" + passed_params): exit()
    if os.system(python_cmd + " show1_latex_rpoB_figure.py" + passed_params): exit()
    if os.system(python_cmd + " show2_latex_rpoB_table.py" + passed_params): exit()
    #if os.system(python_cmd + " show3_show_comp_rels.py" + passed_params): exit()
    t1_time = time.time()
    os.chdir("../../../")
    logExecutionTime(t1_time - t0_time, "comp_mutations")


def preparebroad(parameters):
    os.chdir(os.path.abspath("src/prebroad/prepare/"))

    params = ["D", "REVORDER", "SCORES", "OW", "W", "GWAMAR_DATASETS", 
     "RAXML_TYPE", "RAXML_REP", "TREE_SOFT"]
    passed_params = createCommandParams(params, parameters)

    t0_time = time.time()
    if os.system(python_cmd + " p1_download_files.py" + passed_params): exit()
    if os.system(python_cmd + " p2_save_strains.py" + passed_params): exit()
    if os.system(python_cmd + " p3_save_resdata.py" + passed_params): exit()
    if os.system(python_cmd + " p4_combine_resistance_profiles.py" + passed_params): exit()
    if os.system(python_cmd + " p5_save_point_mutations_aa.py" + passed_params): exit()
    t1_time = time.time()
    os.chdir("../../../")
    logExecutionTime(t1_time - t0_time, "preparebroad")

def preparebroadtree(parameters):
    os.chdir(os.path.abspath("src/prebroad/tree/"))

    params = ["D", "REVORDER", "SCORES", "OW", "W", "GWAMAR_DATASETS", 
     "RAXML_TYPE", "RAXML_REP", "TREE_SOFT"]
    passed_params = createCommandParams(params, parameters)

    t0_time = time.time()
    if os.system(python_cmd + " t0_save_point_mutations_nt.py" + passed_params): exit()
    if os.system(python_cmd + " t1_prepare_alignments.py" + passed_params): exit()
    if os.system(python_cmd + " t2_compute_subset_tree.py" + passed_params): exit()
    if os.system(python_cmd + " t3_resolve_clusters.py" + passed_params): exit()
    if os.system(python_cmd + " t4_save_strains_ordered.py" + passed_params): exit()
    t1_time = time.time()
    os.chdir("../../../")
    logExecutionTime(t1_time - t0_time, "preparebroadtree")

def print_help(parameters):
    print("You have not specified the -a parameter or specified it as =h (help)")
    print("For more help, please visit "+"http://bioputer.mimuw.edu.pl/gwamar/")

if __name__ == '__main__':
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    WORKERS = int(parameters["W"])

    python_cmd = sys.executable
    action = parameters["A"].upper()
    
    if action == "P":
        datasets = parameters.get("D").split(",")
        for dataset in datasets:
            parameters["D"] = dataset
            prepare(parameters)
    elif action in ["H", "HELP"]:
        print_help(parameters)
    elif action == "O":
        other(parameters)
    elif action == "S":
        datasets = parameters.get("D").split(",")
        for dataset in datasets:
            parameters["D"] = dataset
            scores_std(parameters)
    elif action == "SR":
        datasets = parameters.get("D").split(",")
        for dataset in datasets:
            parameters["D"] = dataset
            scores_ranks(parameters)
    elif action == "SN":
        scores_norm(parameters)
    elif action == "SP":
        scores_pvalues(parameters)
    elif action == "PV":
        perm_test(parameters)
    elif action in ["A1", "AS1"]:
        datasets = parameters.get("D").split(",")
        for dataset in datasets:
            parameters["D"] = dataset
            analysis1(parameters)
    elif action in ["A2", "AS2"]:
        datasets = parameters.get("D").split(",")
        for dataset in datasets:
            parameters["D"] = dataset
            analysis2(parameters)
    elif action in ["A", "AS"]:
        datasets = parameters.get("D").split(",")
        for dataset in datasets:
            parameters["D"] = dataset
            analysis1(parameters)
            analysis2(parameters)
    elif action in ["C", "CMP"]:
        datasets = parameters.get("D").split(",")
        for dataset in datasets:
            parameters["D"] = dataset
            comparison1(parameters)
            comparison2(parameters)
            comparison3(parameters)
        parameters["D"] = ",".join(datasets)
        comparisonF(parameters)
    elif action in ["C1", "CMP1"]:
        datasets = list(parameters.get("D").split(","))
        for dataset in datasets:
            parameters["D"] = dataset
            comparison1(parameters)
    elif action in ["C2", "CMP2"]:
        datasets = list(parameters.get("D").split(","))
        for dataset in datasets:
            parameters["D"] = dataset
            comparison2(parameters)
    elif action in ["C2", "CMP3"]:
        datasets = list(parameters.get("D").split(","))
        for dataset in datasets:
            parameters["D"] = dataset
            comparison3(parameters)
    elif action in ["CF", "CMPF"]:
        comparisonF(parameters)
    elif action == "CM":
        comp_mutations(parameters)
    elif action == "PB":
        preparebroad(parameters)
        preparebroadtree(parameters)
    elif action == "PBT":
        preparebroadtree(parameters)
    else:
        print("Unknown action: " + action);
