import sys
import os
import copy

from multiprocessing import Pool
from ressoft.utils.file_utils import ensure_dir
try:
    from queue import Queue
except ImportError:
    from Queue import Queue

sys.path.append("../../")
from ressoft.utils.pvalue_utlis import hyperTestPvalue

from ressoft.utils.stats_utils import *
from ressoft.utils.generator import choose
from ressoft.utils.pred_utils import *
from ressoft.utils.input_utils import *
from ressoft.utils.names_utils import *
from ressoft.utils.output import *

def readParameters(param_tab):
    parameters = {}
    for param_el in param_tab:
        tokens = param_el.split("=")
        if len(tokens)==2:
            key = tokens[0]
            value = tokens[1]
            parameters[key] = value
        
    return parameters

def coloredPredictions2(res_profiles, output_format_fn):
    output_format_fh = open(output_format_fn, "w")
    
    output_format_fh.write('<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">\n')
    output_format_fh.write("<head>\n")
    output_format_fh.write('<script src="sorttable.js" type="text/javascript"> </script>\n')
    output_format_fh.write('<script src="jquery-1.7.1.js" type="text/javascript"> </script>\n')
    output_format_fh.write('<script type="text/javascript">\n')
    output_format_fh.write('$(document).ready(function(){\n')
    output_format_fh.write('init();\n')
    output_format_fh.write('});\n')
    output_format_fh.write('</script>\n')
    output_format_fh.write('<link rel="stylesheet" type="text/css" href="vert_text.css" />\n')
    output_format_fh.write("</head>\n")
    output_format_fh.write("<body>\n")
    output_format_fh.write('<table>\n')
    output_format_fh.write('<th> </th>')

    for res_profile in res_profiles:
        drug_name = res_profile.drug_name
        output_format_fh.write('<th style="width:20px"><div class="verticalText">'+drug_name+'</div></th>')
    
    for strain_index in range(0, strains.count(), 1):
        strain_id = strains.getStrain(strain_index)
        print(strain_id)
        #strain_id = strains.strain_ids[strain_index]
        strain_index = strains.getPosition(strain_id)
        output_format_fh.write('<tr>')
        output_format_fh.write('<td>'+strain_id+"</td>")
        for res_profile in res_profiles:
            if strain_index in res_profile.getResistant():
                if strain_index in res_profile.predicted_strains:
                    output_format_fh.write('<td style="background-color: rgb(255,170,170); width:20px">'+'r'+'</td>')
                else:
                    output_format_fh.write('<td style="background-color: rgb(255,0,0); width:20px">'+'R'+'</td>')
            elif strain_index in res_profile.getSusceptible():
                if strain_index in res_profile.predicted_strains:
                    output_format_fh.write('<td style="background-color: rgb(170,255,170); width:20px">'+'s'+'</td>')
                else:
                    output_format_fh.write('<td style="background-color: rgb(0,255,0); width:20px">'+'S'+'</td>')                
            else:
                output_format_fh.write('<td style="width:20px">'+'?'+'</td>')
    
        output_format_fh.write('</tr>')    

    output_format_fh.write('</table>\n')
    output_format_fh.write("</body>\n")
    output_format_fh.write("</html>\n")
    
    output_format_fh.close()


def coloredPredictions(res_profiles, output_format_fn):
    output_format_fh = open(output_format_fn, "w")
    
    output_format_fh.write('<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">\n')
    output_format_fh.write("<head>\n")
    output_format_fh.write('<script src="sorttable.js" type="text/javascript"> </script>\n')
    output_format_fh.write('<script src="jquery-1.7.1.js" type="text/javascript"> </script>\n')
    output_format_fh.write('<script type="text/javascript">\n')
    output_format_fh.write('$(document).ready(function(){\n')
    output_format_fh.write('init();\n')
    output_format_fh.write('});\n')
    output_format_fh.write('</script>\n')
    output_format_fh.write('<link rel="stylesheet" type="text/css" href="vert_text.css" />\n')
    output_format_fh.write("</head>\n")
    output_format_fh.write("<body>\n")
    output_format_fh.write('<table border-collapse=collapse;>\n')
    #output_format_fh.write('<tr>\n')
    #output_format_fh.write('<img src="tree.png" style="width=80px; height:600px">\n')
    
    
    #output_format_fh.write('<td style="width:100px; border-bottom-color: black; border-left-color:black; border-top-color:black;">\n')
    output_format_fh.write('<td style="width:100px;">\n')
        #output_format_fh.write('</td>\n')
#    output_format_fh.write('<table>\n')
#    output_format_fh.write('<th style="height:100px">\n')
#    output_format_fh.write('</th>\n')
#    output_format_fh.write('<tr>\n')
    #output_format_fh.write('</tr>\n')
    #output_format_fh.write('</table>\n')
    output_format_fh.write('<img src="tree.png" style="margin-top:120px; width=80px; height:1700px">\n')
    output_format_fh.write('</td>\n')
    output_format_fh.write('<td>\n')
    output_format_fh.write('<table  border-collapse=collapse;>\n')
    output_format_fh.write('<tr>\n')
    #output_format_fh.write('<td style="width:150px; border-top-color:white;">Phylo</td>\n')
    output_format_fh.write('<td style="width:150px;">Strain:</td>\n')

    for res_profile in res_profiles:
        drug_name = res_profile.drug_name
        output_format_fh.write('<th style="width:20px"><div class="verticalText">'+drug_name+'</div></th>')
    
    print("LEN", len(strains.strain_ids))
    for strain_index in range(0, len(strains.strain_ids), 1):
        
        strain_id = strains.getStrain(strain_index)
        print(strain_id)
        #strain_id = strains.strain_ids[strain_index]
        strain_index = strains.getPosition(strain_id)
        output_format_fh.write('<tr>')
        output_format_fh.write('<td>'+strain_id+"</td>")
        for res_profile in res_profiles:
            if strain_index in res_profile.getResistant():
                if strain_index in res_profile.predicted_strains:
                    output_format_fh.write('<td style="background-color: rgb(255,170,170); width:20px">'+'r'+'</td>')
                else:
                    output_format_fh.write('<td style="background-color: rgb(255,0,0); width:20px">'+'R'+'</td>')
            elif strain_index in res_profile.getSusceptible():
                if strain_index in res_profile.predicted_strains:
                    output_format_fh.write('<td style="background-color: rgb(170,255,170); width:20px">'+'s'+'</td>')
                else:
                    output_format_fh.write('<td style="background-color: rgb(0,255,0); width:20px">'+'S'+'</td>')                
            else:
                output_format_fh.write('<td style="width:20px">'+'?'+'</td>')
    
        output_format_fh.write('</tr>')    
    #for drug_name in res_profiles:
    #    drug_res_profile = res_profiles[drug_name]
    #    
    #    
    #    output_format_fh.write(drug_name)
    output_format_fh.write('</td>\n')
    output_format_fh.write('</tr>\n')
    output_format_fh.write('</table>\n')
    output_format_fh.write("</body>\n")
    output_format_fh.write("</html>\n")
    
    output_format_fh.close()


    

species = "mtb80"
input_dir = os.path.abspath("../../../data_input/"+species+"_res/")+"/"
output_dir = os.path.abspath("../../../data_output/"+species+"_html/")+"/"

ensure_dir(output_dir)

input_fh = open(input_dir + "strains-all.txt")
strains = readStrains(input_fh)
input_fh.close()

input_fh = open(input_dir + "res_profiles_ret.txt")
res_profiles = readResistanceProfiles(input_fh, strains)
input_fh.close()

print(input_dir)

input_fh = open(input_dir + "pred_res_profiles.txt")
pred_res_profiles = readResistanceProfiles(input_fh, strains)
input_fh.close()


#strains_ord_fh = open(input_dir + "names-strains-ord.txt")
#strains_ord = readStrainNamesOrd(strains_ord_fh)
#strains_ord_fh.close()
    
    
output_format_fn = output_dir + "colored_res_profiles2.html"
coloredPredictions(res_profiles, output_format_fn)
pred_output_format_fn = output_dir + "colored_pred_res_profiles2.html"
coloredPredictions(pred_res_profiles, pred_output_format_fn)
        
        
        