import sys
import os
import copy
from multiprocessing import Pool


sys.path.append("../../../../camber2/src/")
sys.path.append("../../")

from soft.utils.camber_params_utils import *
from ressoft.utils.file_utils import ensure_dir
from soft.utils.camber_utils import *
from ressoft.utils.stats_utils import *
from ressoft.utils.generator import choose
from ressoft.utils.pred_utils import *
from ressoft.utils.input_utils import *
from ressoft.utils.names_utils import *
from ressoft.utils.output import *
from ressoft.modelling.model_logic import *

def printScoresGene(genes_to_show, drug_name, output_det_fh, only_mutated = False):
    sorted_gene_profiles = sorted(genes_to_show, key=lambda gene_profile:(gene_profile.gene_id, gene_profile.start))

    output_det_fh.write("gene_id\tdesc\tposition\ttree_score\twsupport\tsupport\tor\n")
    
    genes_map = {}
    
    for gene_profile in sorted_gene_profiles:
        if not gene_profile.gene_id in genes_map:
            genes_map[gene_profile.gene_id] = []
        genes_map[gene_profile.gene_id].append(gene_profile)

        
    for gene_id in genes_map.keys():
        last_pos = -1
        for gene_profile in genes_map[gene_id]:
            gene_profile_id = gene_profile.gene_profile_id
            gene_id = gene_profile.gene_id
            
            tokens = gene_profile_id.split("-")
            if len(tokens) == 2:
                gene_id_text = tokens[1]
            else:
                gene_id_text = tokens[0]
            gene_profile.setGeneIDText(gene_id_text)
            
            if only_mutated == False:
                while last_pos + 1 < gene_profile.start:
                    text_line = gene_profile.gene_name + "\t"
                    text_line += "no" + "\t"
                    text_line += str(last_pos + 1) + "\t"
                    text_line += str("%.2f" % 0) + "\t"
                    text_line += str("%.2f" % 0) + "\t"
                    text_line += str("%.2f" % 0) + "\t"
                    text_line += str("%.2f" % 0) + "\t"
                    output_det_fh.write(text_line + "\n")
                    last_pos += 1 
    
            text_line = gene_profile.gene_name + "\t"
            text_line += gene_profile.desc + "\t"
            text_line += str(gene_profile.start) + "\t"
            text_line += str("%.2f" % gene_profile.tscore) + "\t"
            text_line += str("%.2f" % gene_profile.wsupport) + "\t"
            text_line += str("%.2f" % gene_profile.support) + "\t"
            text_line += str("%.2f" % gene_profile.odds_support) + "\t"
            output_det_fh.write(text_line + "\n")
    return None
        
def printHtmlStephen(genes_to_show, drug_name, strains, output_det_fh, show_profiles = True, hidden_res_profile = None, drug_res_genes=None, drug_targets=None, strain_index = None, predicted_res_profile = None, gene_functions = None, type="G", mutabilities=None):
    sorted_gene_profiles = sorted(genes_to_show, key=lambda gene_profile:(gene_profile.gene_id, gene_profile.start))
    output_det_fh.write('<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">\n')
    output_det_fh.write("<head>\n")
    output_det_fh.write('<script src="sorttable.js" type="text/javascript"> </script>\n')
    output_det_fh.write('<script src="jquery-1.7.1.js" type="text/javascript"> </script>\n')
    output_det_fh.write('<script src="row_expand.js" type="text/javascript"> </script>\n')
    output_det_fh.write('<script type="text/javascript">\n')
    output_det_fh.write('$(document).ready(function(){\n')
    output_det_fh.write('init();\n')
    output_det_fh.write('});\n')
    output_det_fh.write('</script>\n')
    output_det_fh.write('<link rel="stylesheet" type="text/css" href="vert_text.css" />\n')
    output_det_fh.write("</head>\n")
    output_det_fh.write("<body>\n")
    output_det_fh.write('<table>\n')
    output_det_fh.write('<tr>\n')
    output_det_fh.write('<th style="width:100px">Gene identifier</th>\n')

    if len(sorted_gene_profiles) > 0:
        if sorted_gene_profiles[0].start != None:
            output_det_fh.write('<th style="width:40px">Position</th>\n')
        if sorted_gene_profiles[0].desc != None and sorted_gene_profiles[0].desc != "":
            output_det_fh.write('<th style="width:40px">Description</th>\n')
    
    if type == "S":
        if gene_functions != None: 
            output_det_fh.write('<th style="width:200px">Gene functional annotation</th>\n')
        if mutabilities != None:
            output_det_fh.write('<th style="width:200px">Gene mutabilities/th>\n')            
            
        output_det_fh.write('<th style="width:40px">Weighted support</th>\n')
        output_det_fh.write('<th style="width:40px">Tree score</th>\n')
        output_det_fh.write('<th style="width:40px">Odds ratio</th>\n')
        output_det_fh.write('<th style="width:40px">Drug-resistance determinant?</th>\n')
    else:        
        if gene_functions != None:
            output_det_fh.write('<th style="width:200px">Gene functional annotation</th>\n')
        if mutabilities != None: 
            output_det_fh.write('<th style="width:200px">Gene mutabilities/th>\n')     
        output_det_fh.write('<th style="width:40px">Weighted support</th>\n')
        output_det_fh.write('<th style="width:40px">Tree score</th>\n')
        output_det_fh.write('<th style="width:40px">Odds ratio</th>\n')
        output_det_fh.write('<th style="width:40px">Drug-resistance determinant?</th>\n')

    if hidden_res_profile != None:
        for strain_index in range(0, strains.count(), 1):
            strain_id = strains.getStrain(strain_index)
            strain_id = strain_id.strip()
            if strain_index in hidden_res_profile.getSusceptible():
                if strain_index in hidden_res_profile.predicted_strains or strain_index in hidden_res_profile.hidden_strains:
                    output_det_fh.write('<th style="background-color: rgb(170,255,170); width:65px">'+strain_id+' (s)</th>')
                else:
                    output_det_fh.write('<th style="background-color: rgb(0,255,0); width:65px">'+strain_id+' (S)</th>')
            elif strain_index in hidden_res_profile.getResistant():
                if strain_index in hidden_res_profile.predicted_strains or strain_index in hidden_res_profile.hidden_strains:
                    output_det_fh.write('<th style="background-color: rgb(255,170,170); width:65px">'+strain_id+' (r)</th>')
                else:
                    output_det_fh.write('<th style="background-color: rgb(255,0,0); width:65px">'+strain_id+' (R)</th>') 
            else:
                output_det_fh.write('<td style="width:65px">'+strain_id+' (?)</td>') 

    output_det_fh.write('</tr>\n')    
    
    count = 0
    for gene_profile in sorted_gene_profiles:
        count += 1
        gene_profile_id = gene_profile.gene_profile_id
        gene_id = gene_profile.gene_id
        
        tokens = gene_profile_id.split("-")
        if len(tokens) == 2:
            gene_id_text = tokens[1]
        else:
            gene_id_text = tokens[0]
        gene_profile.setGeneIDText(gene_id_text)

        output_det_fh.write('<tr>\n')
        if type == "S":        
            if gene_profile.gene_id != gene_profile.gene_name:
                output_det_fh.write('<td style="width:200px">'+gene_profile.gene_id+"("+gene_profile.gene_name+")"+'</td>')
            else:
                output_det_fh.write('<td style="width:200px">'+gene_profile.gene_id+"("+gene_profile.gene_name+")"+'</td>')                
            if gene_profile.start != gene_profile.end:
                output_det_fh.write('<td style="width:200px">'+"["+str(gene_profile.start)+":"+str(gene_profile.end)+"]"+'</td>')
            else:
                output_det_fh.write('<td style="width:200px">'+str(gene_profile.start)+'</td>')
        else:
            output_det_fh.write('<td style="width:200px">'+gene_profile.gene_id_text+'</td>')
            
            if gene_profile.start != None:
                output_det_fh.write('<td style="width:20px">'+str(gene_profile.start)+'</td>\n')
        if gene_profile.desc != None and gene_profile.desc != "":
            output_det_fh.write('<td style="width:20px">'+str(gene_profile.desc)+'</td>\n')
                
        if gene_functions != None:
            if gene_id in gene_functions:
                output_det_fh.write('<td style="width:200px">'+gene_functions[gene_id]+'</td>')
            else:
                output_det_fh.write('<td style="width:200px">unknown</td>')
        if mutabilities != None:
            if gene_id in mutabilities:
                output_det_fh.write('<td style="width:200px">'+str("%.8f" % mutabilities[gene_id])+' </td>')
            else:
                output_det_fh.write('<td style="width:200px">NA</td>')
             
        output_det_fh.write('<td style="width:20px; border-left-width: 3px;">'+str(gene_profile.wsupport).replace(".",",")+'</td>')
        output_det_fh.write('<td style="width:20px">'+str(gene_profile.tscore).replace(".",",")+'</td>')
        output_det_fh.write('<td style="width:20px">'+str(gene_profile.odds_support).replace(".",",")+'</td>')
  #      output_det_fh.write('<td style="width:20px">'+str(gene_profile.pvalue_wsupport).replace(".",",")+'</td>')
  #      output_det_fh.write('<td style="width:20px">'+str(gene_profile.pvalue_wsupport_corr).replace(".",",")+'</td>')
        
        is_det = False
        #print(drug_targets)
        #print(drug_res_genes)
        if (drug_targets != None and gene_profile in drug_targets) or (drug_res_genes != None and gene_profile in drug_res_genes):
            is_det = True
#        if drug_targets != None and gene_id in drug_targets:
#            (start, end) = drug_targets[gene_id]
#            (s, e) = gene_profile.positionRange()
#            if s>=start and e<=end:
#                is_det = True
#        if drug_res_genes != None and gene_id in drug_res_genes:
#            is_det = True

        if is_det:
            output_det_fh.write('<td style="width:20px">YES</td>')
        else:
            output_det_fh.write('<td style="width:20px">NO</td>')


        #output_det_fh.write('<td style="width:20px">'+gene_profile.diff_type+'</td>')
        for strain_id in range(0, strains.count(), 1):
            if gene_profile.getDescState(strain_id) == "?":
                if gene_profile.getBinState(strain_id) == '1':
                    output_det_fh.write('<td style="background-color: rgb(230,180,180); width:20px">'+"1:"+str(gene_profile.getState(strain_id))+'</td>')
                elif gene_profile.getBinState(strain_id) == '0':
                    output_det_fh.write('<td style="background-color: rgb(140,230,190); width:20px">'+"0:"+str(gene_profile.getState(strain_id))+'</td>')                
                else:
                    output_det_fh.write('<td style="background-color: rgb(200,200,190); width:20px">'+"0:"+str(gene_profile.getState(strain_id))+'</td>')
            else:
                if gene_profile.getBinState(strain_id) == '1':
                    output_det_fh.write('<td style="background-color: rgb(230,180,180); width:20px">'+"1:"+str(gene_profile.getState(strain_id))+":"+str(gene_profile.getDescState(strain_id))+'</td>')
                elif gene_profile.getBinState(strain_id) == '0':
                    output_det_fh.write('<td style="background-color: rgb(140,230,190); width:20px">'+"0:"+str(gene_profile.getState(strain_id))+":"+str(gene_profile.getDescState(strain_id))+'</td>')                
                else:
                    output_det_fh.write('<td style="background-color: rgb(200,200,190); width:20px">'+"0:"+str(gene_profile.getState(strain_id))+":"+str(gene_profile.getDescState(strain_id))+'</td>')
        output_det_fh.write('</tr>\n')        
        

    output_det_fh.write('</table>\n')
    output_det_fh.write("</body>\n")
    output_det_fh.write("</html>\n")

def scoreMutations(drug_res_profile, drug_name, drug_targets, drug_res_genes, tree, all_mutations, gene_presence, strains, output_format_fh=None, parameters=None):
    species = parameters["SPECIES"] 
    details = parameters["DETAILS"]
    subset = parameters["SUBSET"]
    hide = int(parameters["HIDE"]) 
    beta = int(parameters["BETA"]) 
    pred = parameters["PRED"]
    theta = float(parameters["THETA"])
    dist = int(parameters["DIST"])
    zeta = float(parameters["ZETA"])
    tscore_type = parameters["TSCORE_TYPE"]
    exp_prefix = parameters["EXP_PREFIX"]
    grouping_type = parameters["GROUPING"] 
    m_type = parameters["TYPE"]
         
    input_dir = os.path.abspath("../../../data_input/"+species+"_res/")+"/"
    output_dir = os.path.abspath("../../../data_output/"+species+"_out/")+"/"

    if beta == -1:
        beta_str = str("%.2f" % float(len(drug_res_profile.getResistant())/float(len(drug_res_profile.getSusceptible()))))
        beta = float(beta_str)

    tree.setObservedResProfile(drug_res_profile.full_profile)
    calcSubleavesResCount(tree)
    calcSubnodesCount(tree)

    newtree = removeUnResLeaves(tree)
     
    calcSubleavesResCount(newtree)
    calcSubnodesCount(newtree)
    calcDetParsResModel(newtree)
    newtree.calcContributions(drug_res_profile, beta)

    calcRefSatesMutations(all_mutations, drug_res_profile)
    calcBinSatesMutations(all_mutations, drug_res_profile)
    calcGenesCoverSupports(all_mutations, drug_res_profile, beta)

    con_mutations, neu_mutations, ess_mutations = categorizeMultiMutations(all_mutations, drug_res_profile)
    imp_mutations = con_mutations + neu_mutations + ess_mutations
    gene_profiles = geneMutations(imp_mutations)
    con_gene_profiles = geneMutations(con_mutations)
    
    gene_profiles = groupGeneMutations(gene_profiles, dist, con_gene_profiles, grouping_type)
    print("AAA", len(gene_profiles))
    
    computeGeneProfiles(gene_profiles)
    computeGeneProfilesEss(gene_profiles)
    
    calcGenesCoverSupports(gene_profiles, drug_res_profile, beta)
    calcGenesWeightedSupports(gene_profiles, drug_res_profile, newtree, beta)
    calcGenesTreeScore(gene_profiles, drug_res_profile, newtree, theta=theta, zeta=zeta)
    calcOddsRatio(gene_profiles, drug_res_profile)
    
    for gene_profile in gene_profiles:
        muts_sorted = sorted(gene_profile.mutations, key=lambda mutation:mutation.tscore)
        gene_profile.start = muts_sorted[0].position
        gene_profile.end = muts_sorted[0].position
        gene_profile.desc = muts_sorted[0].desc
        
    
    if grouping_type == "G":
        gene_profiles_sorted = sorted(gene_profiles, key=lambda gp:-gp.best_mutation.wsupport)
        genes_to_show = list(gene_profiles_sorted)[:7000]        
    elif details == "N":
        genes_to_show = filerGenesToShow(gene_profiles, drug_res_profile, drug_res_genes=drug_res_genes, drug_targets=drug_targets)
    elif details == "H":
        genes_to_show = filerGenesToShowHigh(gene_profiles, max_support, max_support2=max_support2, drug_res_genes=drug_res_genes, drug_targets=drug_targets)
    elif details == "Y":
        genes_to_show = filerGenesToShowBySupport(gene_profiles, 1, drug_res_genes=drug_res_genes, target_genes=drug_targets)
    elif details == "S":
        gene_profiles_sorted = sorted(gene_profiles, key=lambda gp:-gp.wsupport)
        genes_to_show = list(gene_profiles_sorted)[:7000]

    
    output_res_fn = output_dir +"/html_subset/"+str(exp_prefix)+"-"+str(hide)+"-"+str(dist)+"-"+grouping_type+"-"+details+"-"+subset+"-"+m_type+"-"+pred+"-"+str(beta)+"-"+str(theta)+"-"+str(zeta)+"-"+str(tscore_type)+"-"+drug_name+".html"
    output_fh = open(output_res_fn, "w")            
    printHtmlStephen(genes_to_show, drug_name, strains, output_fh, show_profiles=True, hidden_res_profile=drug_res_profile, drug_res_genes=drug_res_genes, drug_targets=None, gene_functions=None, type=m_type, mutabilities=None)
    output_fh.close()

    output_res_fn = output_dir +"/r_subset/"+drug_name+".txt"
    output_fh = open(output_res_fn, "w")            
    printScoresGene(genes_to_show, drug_name, output_fh, only_mutated = False)
    output_fh.close()


    return None


def evaluateMethod(drug_name, res_profiles, drug_targets, drug_res_genes, tree, all_mutations, gene_presence, strains, parameters):
    species = parameters["SPECIES"] 
    details = parameters["DETAILS"]
    subset = parameters["SUBSET"]
    hide = int(parameters["HIDE"]) 
    theta = parameters["THETA"]
    beta = int(parameters["BETA"])  
    pred = parameters["PRED"]
    zeta = float(parameters["ZETA"])
    tscore_type = parameters["TSCORE_TYPE"]
    dist = int(parameters["DIST"])
    exp_prefix = parameters["EXP_PREFIX"]
    grouping_type = parameters["GROUPING"] 
    m_type = parameters["TYPE"] 
    
    input_dir = os.path.abspath("../../../data_input/"+species+"_res/")+"/"
    output_dir = os.path.abspath("../../../data_output/"+species+"_out/")+"/"
        
    drug_res_profile = copy.deepcopy(res_profiles[drug_name])
    scoreMutations(drug_res_profile, drug_name, drug_targets[drug_name], drug_res_genes[drug_name], tree, all_mutations, gene_presence, strains, parameters=parameters)
#     output_format_fh.close()

def scoreDrug(parameters):
    species = parameters["SPECIES"] 
    subset = parameters["SUBSET"]
    m_type = (parameters["TYPE"])
    pred = (parameters["PRED"])
    drug_name = parameters["DRUG_NAME"]

    input_dir = os.path.abspath("../../../data_input/"+species+"_res/")+"/"
    output_dir = os.path.abspath("../../../data_output/"+species+"_out/")+"/"

    input_fh = open(input_dir + "strains-all.txt")
    strains = readStrains(input_fh)
    input_fh.close()
    
    if pred == "B":
        input_fn = input_dir + "pred_res_profiles.txt"
    else:
        input_fn = input_dir + "pred_res_profiles.txt"
    input_fh = open(input_fn)
    res_profiles_list = readResistanceProfiles(input_fh, strains)
    input_fh.close()
    
    res_profiles = {}
    for res_profile in res_profiles_list:
        res_profiles[res_profile.drug_name] = res_profile
    
    cluster_fh = open(input_dir + "conn_comp-det.txt")
    clusters, id_cluster_map = readClusters(cluster_fh, strains)
    cluster_fh.close()
    
    anns_fn = input_dir+"names_anns.txt"
    if os.path.exists(anns_fn):
        anns_fh = open(anns_fn)
        names_mapping = readNamesMapping(anns_fh)
        anns_fh.close()
    else:
        names_mapping = None
    
    strains_ord_fh = open(input_dir + "names-strains-ord.txt")
    strains_ord = readStrainNamesOrd(strains_ord_fh)
    strains_ord_fh.close()
    
    input_fh = open(input_dir + "drug_targets.txt")
    drug_targets = loadDrugTargets(input_fh, clusters, id_cluster_map, strains_ord)
    input_fh.close()
    
    input_fh = open(input_dir + "drug_res_genes.txt")
    drug_res_genes = loadDrugResGenes(input_fh, clusters, id_cluster_map, strains_ord)
    input_fh.close()
    
    print("DRUG RES GENES", drug_res_genes)

    input_fh = open(input_dir + 'phylo_tree.txt')
    tree = readPhyloTree(input_fh, strains)
    input_fh.close()
    
    if subset == "Y":
        input_fh = open(input_dir + 'subset.txt')
        gene_subset = readGeneSubset(input_fh, clusters, id_cluster_map, strains_ord)
        input_fh.close()
    else:
        gene_subset = None
        
 #   gene_subset = set(gene_subset) & set(drug_res_genes[drug_name].keys())
        
    input_fh = open(input_dir + 'man-all-mutations.txt')
    all_mutations, gene_presence = readMultiMutations(input_fh, strains, clusters=clusters, id_cluster_map=id_cluster_map, strains_ord=strains_ord, names_mapping=names_mapping, gene_subset=gene_subset, m_type=m_type, syn = True)
    input_fh.close()
    
    print("READ")
    sys.stdout.flush()
    
    imp_mutations = filterMutations(all_mutations, strains)
    
    
    diffTime("FILTERED")
    print("MUTATIONS: ", len(imp_mutations))
    evaluateMethod(drug_name, res_profiles, drug_targets, drug_res_genes, tree, imp_mutations, gene_presence, strains, parameters)
    diffTime("EVALUATED")
    
    return drug_name
    
if __name__ == '__main__':
    
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    
    if "W" in parameters:
        WORKERS = int(parameters["W"])
    else:
        WORKERS = 4
    #WORKERS = 1
    #species = "sta_plas_184"
    
    if "SPECIES" in parameters and not parameters["SPECIES"] in ["mtb","sta","ba"]:
        species = parameters["SPECIES"]
    else:
        species = "sta100"
        
   # species = "sta_plas_184"
    
    input_dir = os.path.abspath("../../../data_input/"+species+"_res/")+"/"
    output_dir = os.path.abspath("../../../data_output/"+species+"_out/")+"/"
    format_dir = os.path.abspath("../../../data_output/"+species+"_format/")+"/"
    
    ensure_dir(output_dir)
    ensure_dir(output_dir +"/html_subset/")
    ensure_dir(output_dir +"/r_subset/")
    ensure_dir(output_dir + "/det/")
    #scripts_dir = "/research/wongls-group/wozniak/scripts_pred/"
    #scripts_dir = "../../"+species+"-scripts_pred/"
    #ensure_dir(scripts_dir)
    
    cur_dir = os.path.abspath(os.curdir) +"/"
    
    hide_params = set(["0"])
    dist_params = set(["0"])
    pred_params = set(["A"])
    beta_params = set(["-1"])
    grouping_params = set(["N"])
    details_params = set(["S"])
    subset_params = set(["Y"])
    type_params = set(["S"])
    theta_params = ["0.5"]
    zeta_params = ["0.0"]
    tscore_params = ["P"]
    exp_prefix = "FINAL4"
    
    input_fh = open(input_dir + "pred_res_profiles.txt")
    drug_names = readDrugNames(input_fh)
    input_fh.close()
    print(drug_names)
        
    TASKS = []
    
    TASK = {}
    count = 0
    for theta_param in theta_params:
        for beta_param in beta_params:
            for pred_param in pred_params:
                for type_param in type_params:
                    for hide_param in hide_params:
                        for dist_param in dist_params:
                            for tscore_param in  tscore_params:
                                for zeta_param in zeta_params:
                                    for grouping_param in grouping_params:
                                        for details_param in details_params:
                                            for subset_param in subset_params:
                                                for drug_name in drug_names:
                                                    if type_param == "G" and dist_param != "0":
                                                        continue
                                                    if grouping_param == "G" and dist_param != "0":
                                                        continue
                                                    #if drug_name == "Rifampicin":
                                                    #    continue
                                                    TASK["SPECIES"] = species
                                                    TASK["DETAILS"] = str(details_param)
                                                    TASK["HIDE"] = str(hide_param)
                                                    TASK["SUBSET"] = str(subset_param)
                                                    TASK["DIST"] = str(dist_param)
                                                    TASK["GROUPING"] = str(grouping_param)
                                                    TASK["EXP_PREFIX"] = exp_prefix 
                                                    TASK["TYPE"] = type_param
                                                    TASK["PRED"] = pred_param
                                                    TASK["BETA"] = beta_param
                                                    TASK["THETA"] = theta_param
                                                    TASK["TSCORE_TYPE"] = tscore_param
                                                    TASK["ZETA"] = zeta_param  
                                                    TASK["DRUG_NAME"] = drug_name
                                                    #output_det_fn = output_dir + "/det/"+str(exp_prefix)+"-"+str(0)+"-"+str(dist_param)+"-"+grouping_param+"-"+details_param+"-"+subset_param+"-"+type_param+"-"+pred_param+"-"+str(beta_param)+"-"+drug_name+".txt"
        
                                                    TASKS.append(TASK.copy())
            
                                                  #  scoreDrug(TASK)

    for TASK in TASKS:
        print(TASK)
    if WORKERS > 1:                               
        pool = Pool(processes=WORKERS)   
        for r in pool.imap(scoreDrug, TASKS):
            print(r)
    else:
        for T in TASKS:
            scoreDrug(T)

