import sys
import os
import copy
from ete2 import Tree
from ete2 import PhyloTree
from ete2.treeview.main import TreeStyle, NodeStyle, TreeStyle
from multiprocessing import Pool

sys.path.append("../../")

from soft.utils.camber_params_utils import *
from ressoft.utils.file_utils import ensure_dir
from soft.utils.camber_utils import *
from ressoft.utils.stats_utils import *
from ressoft.utils.generator import choose
from ressoft.utils.pred_utils import *
from ressoft.utils.input_utils import *
from ressoft.utils.names_utils import *
from ressoft.utils.output import *
from ressoft.modelling.model_logic import *
from ressoft.modelling.habib import *
from ressoft.utils.pvalue_utlis import hyperTestPvalue
from ressoft.utils.stats_utils import *
from ressoft.utils.generator import choose
from ressoft.utils.pred_utils import *
from ressoft.utils.input_utils import *
from ressoft.utils.names_utils import *
from ressoft.utils.output import *



nstyle = NodeStyle()
#nstyle["shape"] = "sphere"
nstyle["size"] = 10
nstyle["fgcolor"] = "black"

resstyle = NodeStyle()
#resstyle["shape"] = "sphere"
resstyle["size"] = 20
resstyle["fgcolor"] = "red"

suscstyle = NodeStyle()
#nstyle["shape"] = "sphere"
suscstyle["size"] = 20
suscstyle["fgcolor"] = "green"

interstyle = NodeStyle()
#nstyle["shape"] = "sphere"
interstyle["size"] = 20
interstyle["fgcolor"] = "yellow"


def loadDrugsATC(input_fh):
    #drugs_list = []
    drugs_map = {}
    for line in input_fh.readlines():
        if not line:
            continue
        line = line.strip()
        tokens = line.split()
        if len(tokens) == 0:
            continue
        drug_name = tokens[0]
        atc_name = tokens[1]
        if atc_name == "x":
            continue
        if not drug_name in drugs_map:
            drugs_map[drug_name] = atc_name
    drugs_list = sorted(drugs_map, key=lambda dn:drugs_map[dn])
    return drugs_list, drugs_map
    
def loadDrugsFromData(input_fh):
    drugs_list = []
    
    for line in input_fh.readlines():
        if not line:
            continue
        line = line.strip()
        tokens = line.split()
        if len(tokens) > 0:
            drug_name = tokens[0]
            strain_id = tokens[1]
            res_info = tokens[2]
            cite = tokens[3]
            if not drug_name in drugs_list:
                drugs_list.append(drug_name)
    
    return drugs_list

def loadResProfiles(input_fh):
    res_profiles = {}
    
    for line in input_fh.readlines():
        if not line:
            continue
        line = line.strip()
        line = line.split("#")[0]
        tokens = line.split()
        if not len(tokens) in [3,4]:
            continue
        drug_name = tokens[0]
        
        res_profiles[drug_name] = {}
        res_profiles[drug_name]["R"] = set([])
        res_profiles[drug_name]["S"] = set([])
        res_profiles[drug_name]["I"] = set([])
        
        if len(tokens) == 3:
            strains_res_tokens = tokens[1].split(";")
            strains_susc_tokens = tokens[2].split(";")
            
            for strains_res_token in strains_res_tokens:
                strain_id = strains_res_token.strip("*")
                if len(strain_id) > 0:
                    res_profiles[drug_name]["R"].add(strain_id)
            for strains_susc_token in strains_susc_tokens:
                strain_id = strains_susc_token.strip("*")
                if len(strain_id) > 0:
                    res_profiles[drug_name]["S"].add(strain_id)
        elif len(tokens) == 4:
            strains_res_tokens = tokens[1].split(";")
            strains_susc_tokens = tokens[2].split(";")
            strains_int_tokens = tokens[3].split(";")
            
            for strains_res_token in strains_res_tokens:
                strain_id = strains_res_token.strip("*")
                if len(strain_id) > 0:
                    res_profiles[drug_name]["R"].add(strain_id)
            for strains_susc_token in strains_susc_tokens:
                strain_id = strains_susc_token.strip("*")
                if len(strain_id) > 0:
                    res_profiles[drug_name]["S"].add(strain_id)
            for strains_int_token in strains_int_tokens:
                strain_id = strains_int_token.strip("*")
                if len(strain_id) > 0:
                    res_profiles[drug_name]["I"].add(strain_id)            
    return res_profiles

def loadResData(input_fh):
    res_data_map = {}
    
    for line in input_fh.readlines():
        if not line:
            continue
        tokens = line.strip().split("\t")
        
        if len(tokens) == 4:
            try:
                drug_name = tokens[0]
                if len(drug_name) > 0 and drug_name[0] == "#":
                    continue
                
                strain_id = tokens[1].replace("/", "")
                res_info = tokens[2]
                cite = tokens[3]
                if not strain_id in res_data_map:
                    res_data_map[strain_id] = {}
                if not drug_name in res_data_map[strain_id]:
                    res_data_map[strain_id][drug_name] = set([])
                res_data_map[strain_id][drug_name].add((res_info, cite))
            except:
                print("ERROR", line)
        elif len(tokens) == 5:
            
            try:
                drug_name = tokens[0]
                if len(drug_name) > 0 and drug_name[0] == "#":
                    continue
                
                strain_id = tokens[1].replace("/", "")
                res_info = tokens[2] + ":" + tokens[3]
                cite = tokens[4]
                if not strain_id in res_data_map:
                    res_data_map[strain_id] = {}
                if not drug_name in res_data_map[strain_id]:
                    res_data_map[strain_id][drug_name] = set([])
                res_data_map[strain_id][drug_name].add((res_info, cite))
            except:
                print("ERROR", line)   
    return res_data_map

def translateDataIntoResProfiles(res_data):
    res_profiles = {}
    for strain_id in res_data:
        for drug_name in res_data[strain_id]:
            if drug_name != "Penicillin1":
                continue
            res_count = 0 
            susc_count = 0
            int_count = 0 
            unk_count = 0
                
            if not drug_name in res_profiles:
                res_profiles[drug_name] = {}
                res_profiles[drug_name]["R"] = set([])
                res_profiles[drug_name]["S"] = set([])
                res_profiles[drug_name]["I"] = set([])            
            for (drug_res_info, cite_info) in res_data[strain_id][drug_name]:
                tokens = drug_res_info.split(":")
                
                if tokens[0] == "R":
                    res_count += 1 
                elif tokens[0] == "S":
                    susc_count += 1
                elif tokens[0] == "I":
                    int_count += 1
                elif tokens[0] == "U":
                    unk_count += 1
            if res_count > 0 and susc_count == 0:
                res_profiles[drug_name]["R"].add(strain_id)
            elif res_count == 0 and susc_count > 0:
                res_profiles[drug_name]["S"].add(strain_id)
            elif res_count > 0 and susc_count > 0:
                pass
            elif int_count >0:
                res_profiles[drug_name]["I"].add(strain_id)
            elif unk_count > 0:
                pass
            else:
                pass
                                  
    return res_profiles

if __name__ == '__main__':
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    dataset = "em59"
    dataset_dir = os.path.abspath("../camber-pred/datasets/"+dataset)+"/"

    input_fh = open(dataset_dir + "strain_profiles_ord.txt")
    strains = readStrains(input_fh)
    input_fh.close()

    print(strains.strain_ids)

    #res_profiles = translateDataIntoResProfiles(res_data)
    
  #  drug_name = "Cefotaxime"
  
    input_fh = open(dataset_dir + 'phylo_tree.txt')
    tree = readPhyloTree(input_fh, strains)
    input_fh.close()
    
    strain_dates_map = {}
    
    input_fh = open(dataset_dir + 'res_dates.txt')
    for line in input_fh.readlines():
        tokens = line.split()
        strain_id = tokens[0]
        date = tokens[1]
        strain_dates_map[strain_id] = date
    input_fh.close()
    
    print(strain_dates_map)
   
    output_fh = open(dataset_dir + 'phylo_tree_dates.txt', "w")
    tree_dates = tree.root.toString(lengths=False, strains=strains, is_root=True, strains_map=strain_dates_map)
    output_fh.write(tree_dates + "\n")
    output_fh.close()
   