import sys
import os
import copy
import random
from multiprocessing import Pool
from ete2 import Tree
from ete2 import PhyloTree
from ete2.treeview.main import TreeStyle, NodeStyle, TreeStyle
from ete2 import faces, layouts

sys.path.append("../../")

from soft.utils.camber_params_utils import *
from ressoft.utils.file_utils import ensure_dir
from soft.utils.camber_utils import *
from ressoft.utils.stats_utils import *
from ressoft.utils.generator import choose
from ressoft.utils.pred_utils import *
from ressoft.utils.input_utils import *
from ressoft.utils.names_utils import *
from ressoft.utils.output import *
from ressoft.modelling.model_logic import *
from ressoft.modelling.habib import *
from ressoft.utils.pvalue_utlis import hyperTestPvalue
from ressoft.utils.stats_utils import *
from ressoft.utils.generator import choose
from ressoft.utils.pred_utils import *
from ressoft.utils.input_utils import *
from ressoft.utils.names_utils import *
from ressoft.utils.output import *

nstyle = NodeStyle()
#nstyle["shape"] = "sphere"
nstyle["size"] = 10
nstyle["fgcolor"] = "black"

resstyle = NodeStyle()
#resstyle["shape"] = "sphere"
resstyle["size"] = 20
resstyle["fgcolor"] = "red"

suscstyle = NodeStyle()
#nstyle["shape"] = "sphere"
suscstyle["size"] = 20
suscstyle["fgcolor"] = "green"

dupFace = faces.ImgFace("./Xtr.png")



#suscstyle = NodeStyle()
#suscstyle["shape"] = "sphere"
#suscstyle["size"] = 20
#suscstyle["fgcolor"] = "green"
#suscstyle["hz_line_type"] = 1
#suscstyle["hz_line_color"] = "#cccccc"

ts = TreeStyle()
ts.show_leaf_name = True

def allRanks(parameters):
    dataset = parameters["D"] 
    details = parameters["DETAILS"]
    subset = parameters["SUBSET"]
    hide = int(parameters["HIDE"]) 
    theta = parameters["THETA"]
    beta = int(parameters["BETA"])  
    pred = parameters["PRED"]
    dist = int(parameters["DIST"])
    exp_prefix = parameters["EXP_PREFIX"]
    grouping_type = parameters["GROUPING"] 
    supp_type = parameters["SUPP_TYPE"]
    rt = parameters["RT_PARAM"] 
    m_type = parameters["TYPE"] 
    type_param = parameters["TYPE"] 
    
    dataset_dir = os.path.abspath("../camber-pred/datasets/"+dataset)+"/"

    ensure_dir(dataset_dir + "/det/")
    ensure_dir(dataset_dir + "/ranks/")
    ensure_dir(dataset_dir + "/det_trees/")
    
    input_fh = open(dataset_dir + "strains-all.txt")
    strains = readStrains(input_fh)
    input_fh.close()
    
    input_fh = open(dataset_dir + 'phylo_tree.txt')
    tree = readPhyloTree(input_fh, strains)
    input_fh.close()
    
    input_fh = open(dataset_dir + "res_profiles_ret.txt")
    res_profiles = readResistanceProfiles(input_fh, strains)
    input_fh.close()
    
   # sptree = PhyloTree(species_tree_nw)
    #recon_tree, events = genetree.reconcile(sptree)
    #recon_tree.link_to_alignment(alg)
    
    # Visualize the reconciled tree
    #recon_tree.show()
    #recon_tree.render("phylotree.png", w=2000)

    print("here")

    gene_names = set([])
    for drug_res_profile in res_profiles:
        drug_name = drug_res_profile.drug_name
        print(drug_name)
        if drug_name != "Isoniazid":
            continue
        
        tree.setObservedResProfile(drug_res_profile.full_profile)
        calcSubleavesResCount(tree)
        calcSubnodesCount(tree)
    
        newtree = removeUnResLeaves(tree)    
        
        tree_txt = newtree.root.toString(lengths = False, is_root=True, strains=strains)
        
        genetree = PhyloTree(tree_txt + ";")
        
        for node in genetree.traverse():
            node.add_face(dupFace, 1)
            node.add_face(dupFace, 2)
            node.add_face(dupFace, 3)
            #faces.add_face_to_node(dupFace, node, 0)
            node.set_style(nstyle)

        for strain_index in drug_res_profile.res_strains_ids:
            strain_id = strains.getStrain(strain_index)
            for n in genetree.get_leaves_by_name(strain_id):
                n.set_style(resstyle)
        
        for strain_index in drug_res_profile.sc_strains_ids:
            strain_id = strains.getStrain(strain_index)
            for n in genetree.get_leaves_by_name(strain_id):
                n.set_style(suscstyle)
    
        #genetree.show(tree_style = ts)
                
        input_res_fn = dataset_dir +"/det/"+str(exp_prefix)+"-"+str(hide)+"-"+str(dist)+"-"+grouping_type+"-"+details+"-"+subset+"-"+type_param+"-"+pred+"-"+str(beta)+"-"+str(theta)+"-0.0-P/"+drug_name+".txt"
        if not os.path.exists(input_res_fn) or os.path.getsize(input_res_fn) < 100:
            print("NOT EXIST: ", input_res_fn)
            continue
        input_res_fh = open(input_res_fn)
        print("www")
        
        ret = readFeaturesOne(drug_name, strains, input_res_fh)
        if ret == None:
         #   print("None RETURNED: ", input_res_fn)
            continue
        hidden_strains, gene_profiles, target_genes, drug_res_genes, drug_res_rel = ret
        
        alignments = {}

        for leaf_id in newtree.leaf_ids:
            strain_id = strains.getStrain(leaf_id)
            alignments[strain_id] = ""
        
        gene_profiles_sorted = sorted(gene_profiles, key=lambda gp:-gp.supports_map[supp_type])
        for i in range(0, min(10, len(gene_profiles_sorted)), 1):
            gene_profile = gene_profiles_sorted[i]
            gp_score = gene_profile.supports_map[supp_type]
            
            print("XXXXX", drug_name, gene_profile.gene_name, gene_profile.gene_id)
            
            for strain_index in newtree.leaf_ids:
                strain_id = strains.getStrain(strain_index)
                alignments[strain_id] += gene_profile.full_profile[strain_index]
        
#        
        fasta_txt = ""
        for strain_id in alignments:
            fasta_txt += ">" + strain_id + "\n" + alignments[strain_id] + "\n"
#    
        
        genetree.link_to_alignment(alignment=fasta_txt, alg_format="fasta")
        genetree.render(dataset_dir + "/det_trees/"+str(exp_prefix)+"-"+str(hide)+"-"+str(dist)+"-"+grouping_type+"-"+details+"-"+subset+"-"+type_param+"-"+pred+"-"+str(beta)+"-"+str(theta)+"-0.0-P-"+drug_name+ ".pdf")
        genetree.show()
    return None



if __name__ == '__main__':
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    if "DATASET" in parameters and not parameters["D"] in ["mtb","sta"]:
        dataset = parameters["D"]
    else:
        dataset = "spe"
    dataset = "mtb79"
        
    if "W" in parameters:
        WORKERS = int(parameters["W"])
    else:
        WORKERS = 1

    dataset_dir = os.path.abspath("../camber-pred/datasets/"+dataset)+"/"
    
    ensure_dir(dataset_dir +"/trees/")

    details_param = "S"
    hide_param = 0
    dist_params = ["0"]
    exp_prefix = "FINAL5"
    grouping_params = ["G"]
    subset_param = "N"
    pred_params = ["B"]
    theta_params = ["0.5"]
    type_params = ["S"]
    rt_params = ["T"] 

    TASKS = []
    TASK = {}
    for rt_param in rt_params:
        for grouping_param in grouping_params:            
            for theta_param in theta_params:
                for dist_param in dist_params:
                    for type_param in type_params:
                        if type_param == "G" and dist_param == "20":
                            continue
                        for pred_param in pred_params:
                            for beta_param in range(-1, 0, 1):
                                for supp_type_param in ["tscore"]:
                                    if supp_type_param != "tscore" and theta_param != "0.5":
                                        continue  
                                    TASK["D"] = dataset
                                    TASK["PHASE"] = str(0)
                                    TASK["DETAILS"] = str(details_param)
                                    TASK["HIDE"] = str(hide_param)
                                    TASK["DIST"] = str(dist_param)
                                    TASK["GROUPING"] = str(grouping_param)
                                    TASK["EXP_PREFIX"] = exp_prefix 
                                    TASK["TYPE"] = type_param
                                    TASK["PRED"] = pred_param
                                    TASK["BETA"] = beta_param
                                    TASK["THETA"] = theta_param  
                                    TASK["SUPP_TYPE"] = supp_type_param
                                    TASK["SUBSET"] = subset_param
                                    TASK["RT_PARAM"] = rt_param
                                    
                                    TASKS.append(TASK.copy())
                            
    for TASK in TASKS:
        print(TASK)
        
    if WORKERS > 1:                             
        pool = Pool(processes=WORKERS)   
        for r in pool.imap(allRanks, TASKS):
            print(r)
    else:
        for T in TASKS:
            allRanks(T)
        
    print("END")
    
    
    