import sys
import os
import copy
import random
from multiprocessing import Pool
from ete2 import Tree
from ete2 import PhyloTree
from ete2.treeview.main import TreeStyle, NodeStyle, TreeStyle
from ete2 import faces, layouts

sys.path.append("../../")


mut_faces_map = {}

for i in range(1, 11, 1):
    mut_fn = "../../resources/muts/m"+str(i)+".png"
    if not os.path.exists(mut_fn):
        print("not exists: ", mut_fn)
    mut_faces_map[i] = faces.ImgFace(mut_fn)

print(mut_faces_map)

resstyle = NodeStyle()
resstyle["fgcolor"] = "#ffbeb1"
resstyle["bgcolor"] = "#ffbeb1"

suscstyle = NodeStyle()
suscstyle["fgcolor"] = "lightgreen"
suscstyle["bgcolor"] = "lightgreen"

interstyle = NodeStyle()
interstyle["fgcolor"] = "yellow"
interstyle["bgcolor"] = "yellow"

nameFace = faces.AttrFace("name", fsize=20, fgcolor="#009000")

img_path = "./"
# Create faces based on external images
humanFace = faces.ImgFace(img_path+"human.png")
mouseFace = faces.ImgFace(img_path+"mouse.png")
dogFace = faces.ImgFace(img_path+"dog.png")
chimpFace = faces.ImgFace(img_path+"chimp.png")
fishFace = faces.ImgFace(img_path+"fish.png")
flyFace = faces.ImgFace(img_path+"fly.png")


def mylayout(node):
    # If node is a leaf, add the nodes name and a its scientific
    # name
    if node.is_leaf():
        # Add an static face that handles the node name
        faces.add_face_to_node(nameFace, node, column=0)
        # We can also create faces on the fly
        longNameFace = faces.TextFace(node.name)
        faces.add_face_to_node(longNameFace, node, column=0)

        # text faces support multiline. We add a text face
        # with the whole description of each leaf.
        descFace = faces.TextFace(node.name, fsize=10)
        descFace.margin_top = 10
        descFace.margin_bottom = 10
        descFace.border.margin = 1

        # Note that this faces is added in "aligned" mode
        faces.add_face_to_node(descFace, node, column=0, aligned=True)

        # Sets the style of leaf nodes
        node.img_style["size"] = 12
        node.img_style["shape"] = "circle"
    #If node is an internal node
    else:
        # Sets the style of internal nodes
        pass
#        node.img_style["size"] = 6
#        node.img_style["shape"] = "circle"
#        node.img_style["fgcolor"] = "#000000"

    # If an internal node contains more than 4 leaves, add the
    # images of the represented species sorted in columns of 2
    # images max.
    if len(node)>=4:
        col = 0
        for i, name in enumerate(set(node.get_leaf_names())):
            if i>0 and i%2 == 0:
                col += 1
            # Add the corresponding face to the node
            if name.startswith("Dme"):
                faces.add_face_to_node(flyFace, node, column=col)
            elif name.startswith("Dre"):
                faces.add_face_to_node(fishFace, node, column=col)
            elif name.startswith("Mms"):
                faces.add_face_to_node(mouseFace, node, column=col)
            elif name.startswith("Ptr"):
                faces.add_face_to_node(chimpFace, node, column=col)
            elif name.startswith("Hsa"):
                faces.add_face_to_node(humanFace, node, column=col)
            elif name.startswith("Cfa"):
                faces.add_face_to_node(dogFace, node, column=col)

            # Modifies this node's style
            node.img_style["size"] = 16
            node.img_style["shape"] = "sphere"
            node.img_style["fgcolor"] = "#AA0000"

            
def showTreeMutations(parameters):

    nw = """
    (((Dre:0.008339,Dme:0.300613)1.000000:0.596401,
    (Cfa:0.640858,Hsa:0.753230)1.000000:0.182035)1.000000:0.106234,
    ((Dre:0.271621,Cfa:0.046042)1.000000:0.953250,
    (Hsa:0.061813,Mms:0.110769)1.000000:0.204419)1.000000:0.973467);
    """
    t = Tree(nw)

    ts = TreeStyle()
    ts.layout_fn = mylayout
    t.render("img_faces.png", w=600, tree_style = ts)
    t.show(tree_style = ts)
    
    return None
#    
#    muts = {}
#    input_fn = dataset_dir +"/tree_mapping/"+drug_name+"_muts"+".txt"
#    input_fh = open(input_fn)
#    for line in input_fh.readlines():
#        tokens = line.strip().split()
#        if len(tokens) < 2:
#            continue
#        mut_id = tokens[0]
#        if not mut_id in muts:
#            muts[mut_id] = set([])
#        for i in range(1, len(tokens), 1):
#            muts[mut_id].add(tokens[i])
#        
#    input_fh.close()

#    try:
#        mut_index = 1
#        
#        for mut_id in muts:
#            
#            for leaf_tokens in muts[mut_id]:
#                node = phylotree.get_common_ancestor(leaf_tokens.split(";"))
#                if phylotree.get_tree_root() == node:
#                    nodes_list = phylotree.get_leaves_by_name(leaf_tokens.split(";")[0])
#                    for node in nodes_list:
#                        print("here", node)
#                       # faces.add_face_to_node(mut_faces_map[mut_index], node, column=2)
#                        node.add_face(mut_faces_map[mut_index], 1)
#
#                else:
#                  #  faces.add_face_to_node(mut_faces_map[mut_index], node, column=2)
#                    print("www", node)
#                    node.add_face(mut_faces_map[mut_index], 1)
#           #     faces.add_face_to_node(mut_faces_map[mut_index], phylotree.get_tree_root(), column=1)
#                #phylotree.add_face(mut_faces_map[mut_index], phylotree.get_tree_root(), 1)
#                   
#            mut_index += 1
#
#      #  phylotree.get_tree_root().set_style(resstyle)   
#        
#    except Exception as e:
#        print("exception ! ", e)
#        print(drug_name)
#
#    phylotree.get_tree_root().add_face(faces.ImgFace(mut_fn), 0)
#    phylotree.get_tree_root().set_style(suscstyle)
#
#    ress = set([])
#    input_fn = dataset_dir +"/tree_mapping/"+drug_name+"_res"+".txt"
#    input_fh = open(input_fn)
#    for line in input_fh.readlines():
#        tokens = line.strip().split()
#        if len(tokens) < 2:
#            continue
#        for i in range(0, len(tokens), 1):
#            ress.add(tokens[i])
#    input_fh.close()
#
#    try:
#        for leaf_tokens in ress:
#            node = phylotree.get_common_ancestor(leaf_tokens.split(";"))
#            if phylotree.get_tree_root() == node:
#                nodes_list = phylotree.get_leaves_by_name(leaf_tokens.split(";")[0])
#                for node in nodes_list:
#                    node.set_style(resstyle)
#            else:
#                node.set_style(resstyle)
#    
#
#    except Exception as e:
#        print(e)
#        print(drug_name)
#        


    

    
if __name__ == '__main__':
    
                                                  #  scoreDrug(TASK)
    showTreeMutations({})

