#install.packages()
require(graphics); require(grDevices);

#data_fn = paste(species_id, "_out/r_subset/", drug_name, ".txt", sep = "")
dataset = "datasets/broad_mtb/expP1/analysis/"
data_fn = paste(dataset, "/tbdb_scores_P_all.txt", sep="")
data = read.table(data_fn, header=TRUE, sep="\t")
db_type = "high"

#data2 = data[data[,"drug_name"]!="Pyrazinamide",]
data2 = data
data_all = data2[(data2[,"gene_rel"]=="N" | data2[,db_type] == "Y") & !(data2[,db_type] == "Y" & data2[,"R1"] < 2),]

good_count = nrow(data_all[data_all[,"high"] == "Y" | data_all[,db_type] == "Y",])
neg_count = nrow(data_all[data_all[,"gene_rel"] == "N",])

print(paste("All associations: ", nrow(data_all)))
print(paste("Positives (high confidence): ", sum(data_all[,"high"] == "Y")))
print(paste("Negatives (high confidence): ", sum(data_all[,"high"] == "N")))

score_names = names(data_all)[5:(length(data_all)-9)]
pv_score_names = sort(score_names[substr(score_names,1,2)=="pv"])

scores_desc = c()
x11()
plot(c(), xlab="fpr", ylab="tpr", xlim=c(0.0,1.0), ylim=c(0.0, 1.0))

lwdc = 2.5

n = nrow(data_all)
m = n

scolor = c()
scores_fill = c()

for (score_name in pv_score_names) {
	scolor[[score_name]] <- rgb(runif(1),runif(1),runif(1)) 
	ord = 1.0
	if (substr(score_name,1,1)=="c" | substr(score_name,1,2)=="pv") {
		ord = -1.0
	}
	ranks_sel = data_all[order(ord*data_all[score_name], data_all[db_type]), c(score_name, "all", "high", "gene_rel")]
	
	prec = c()
	recall = c()
	fpr = c()
	tpr = c()
	
	area = 0.0
	area_roc = 0.0
	dist = 1.0
	tp = 0
	fp = 0
	fn = good_count
	tn = neg_count
	
	last_rank = ranks_sel[n, score_name]
	
	for (i in 1:m) {
		rank = ranks_sel[n-i+1, score_name]
		if (ranks_sel[n-i+1, db_type] == "Y" | ranks_sel[n-i+1, "high"] == "Y") {
			tp = tp + 1
			fn = fn - 1
		} else {
			fp = fp + 1
			tn = tn - 1
		}
		
		if (rank != last_rank) {
			if (length(prec) == 0) {
				prec = c(prec, tp / i)
				recall = c(recall, 0.0)
				fpr = c(fpr, 0.0)
				tpr = c(tpr, 0.0)
			} 
			prec = c(prec, tp / (tp + fp))
			recall = c(recall, tp / (tp + fn))
			tpr = c(tpr, tp / (tp + fn))
			fpr = c(fpr, fp / (fp + tn))
			k = length(prec)
			dist_c = sqrt(2*(fp / (fp + tn))^2 + (1-tp / (tp + fn))^2)
			dist = min(dist, dist_c)
			area = area + 0.5*(prec[k] + prec[k-1])*(recall[k]-recall[k-1])
			area_roc = area_roc + 0.5*(tpr[k] + tpr[k-1])*(fpr[k]-fpr[k-1])
		}
		last_rank = ranks_sel[n-i+1, score_name]
	}
	
	prec = c(prec, tp / (tp + fp))
	recall = c(recall, tp / (tp + fn))
	tpr = c(tpr, tp / (tp + fn))
	fpr = c(fpr, fp / (fp + tn))
	
	k = length(tpr)
	if (k > 1) {
		area_roc = area_roc + 0.5*(tpr[k] + tpr[k-1])*(fpr[k]-fpr[k-1])
		#print(paste(score_name, area))
		#print(paste(score_name, area_roc))
		lwdc = lwdc - 0.05
		scores_fill = c(scores_fill, scolor[[score_name]])
		scores_desc = c(scores_desc, paste(score_name, " [AUC=" , round(area_roc,3) , "]", " [AUC=" , round(area,3) , "]", " [DIST=" , round(dist,3) , "]", sep=""))
		points(tpr~fpr, col=scolor[[score_name]], pch=18, lwd=lwdc, type="l", xlim=c(0.0,1.0), ylim=c(0.0, 1.0))
		#points(prec~recall, col=scolor[[score_name]], pch=18, lwd=lwdc, type="l", xlim=c(0.0,1.0), ylim=c(0.0, 1.0))
	}
}

legend("bottomright", fill=scores_fill, legend=scores_desc, y.intersp=1, ncol=2)

