import sys
import os
import shutil
import multiprocessing
from multiprocessing import Pool

sys.path.append("../../")

from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_tree_io_utils import readPhyloTrees
from drsoft.utils.gwamar_bin_utils import *
from drsoft.utils.gwamar_utils import ensure_dir
from drsoft.utils.gwamar_progress_utils import GWAMARProgress
from drsoft.utils.gwamar_ann_utils import readClusterGeneIDs
from drsoft.utils.gwamar_muts_utils import *
from drsoft.utils.gwamar_res_utils import compareProfiles
from drsoft.utils.gwamar_res_io_utils import readResistanceProfiles
from drsoft.utils.gwamar_muts_io_utils import *
from drsoft.utils.gwamar_strains_io_utils import readStrains

def printGoldShort(gold_assocs, comment=""):
    total = 0
    for drug_name in gold_assocs:
        tmp = gold_assocs[drug_name].keys()
        tl = comment+" " +drug_name +" "
        for gene_id in tmp:
            count = len(gold_assocs[drug_name][gene_id])
            total += count
            tl += gene_id +"("+str(count)+"),"

        print(tl.strip(","))
    print(comment +" total: " + str(total))

def readGoldAssociations(input_fh, cluster_ids_rev):
    gold_assocs = {}
    drug_name = ""
    gene_id = ""

    lines = input_fh.readlines()
    for line in lines:
        line = line.strip()
        line = line.split("#")[0].strip()
        if len(line) == 0:
            continue
        if line.startswith("%"):
            drug_name = line[1:]
            gold_assocs[drug_name] = {}
        elif line.startswith(">"):
            tmpline = line[1:]
            tmptokens = tmpline.split()
            if len(tmptokens) == 1:
                gene_id = tmptokens[0]
            else:
                gene_id = tmptokens[1]
            gold_assocs[drug_name][gene_id] = {}
        else:

            tokens = line.split()
            if len(tokens) < 3:
                continue
            position = int(tokens[0].split("|")[0])
            ref_state = tokens[1]
            mut_states = tokens[2].split(",")
            for mut_state in mut_states:
                if not position in gold_assocs[drug_name][gene_id]:
                    gold_assocs[drug_name][gene_id][position] = []
                gold_assocs[drug_name][gene_id][position].append((ref_state, mut_state))
    return gold_assocs

if __name__ == '__main__':
    print("show0_save_associations.py: shows gold association lists.")

    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    dataset = parameters["D"]
    input_dir = parameters["DATASET_INPUT_DIR"]
    gold_dir = parameters["DATASET_GOLD_DIR"]
    results_dir = parameters["RESULTS_DIR"]
    scores_dir = parameters["RESULTS_SCORES_DIR"]
    ranks_dir = parameters["RESULTS_RANKS_DIR"]
    cmp_dir = parameters["RESULTS_CMP_DIR"]
    cmp_assocs_dir = parameters["RESULTS_CMP_ASSOCS_DIR"]

    WORKERS = min(int(parameters["W"]), multiprocessing.cpu_count())
    
    scores_p = parameters["SCORES"].split(",")
    
    input_fh = open(input_dir + "/strains_ordered.txt")
    strains = readStrains(input_fh)
    input_fh.close()

    m_type = parameters["MT"]
    
    input_fh = open(results_dir + "/res_profiles.txt")
    drug_res_profiles = readResistanceProfiles(input_fh, load_int=True)
    input_fh.close()
    
    input_fh = open(results_dir + 'bin_profiles_' + m_type + '_det.txt')
    bin_profile_details = readBinProfileDetails(input_fh, strains.count(), mut_type=m_type, out_format="tuple")
    input_fh.close()

    input_fh = open(results_dir + 'bin_profiles_' + m_type + '.txt')
    bin_profiles = readBinProfiles(input_fh, strains.count(), gen_bin_profiles=True)
    input_fh.close()
    
    input_fn = input_dir + "/cluster_gene_ref_ids.txt"
    if os.path.exists(input_fn):
        cluster_fh = open(input_dir + "/cluster_gene_ref_ids.txt")
        cluster_ids, cluster_ids_rev = readClusterGeneIDs(cluster_fh, gene_ref_ids=True)
        cluster_fh.close()
    else:
        cluster_ids = {} 
        cluster_ids_rev = {}

    input_fn = gold_dir + "/gold_assocs_H.txt"
    if not os.path.exists(input_fn):
        print("File does not exist:" + input_fn)
        drug_res_conf_db = {}
    else:
        input_fh = open(input_fn)
        drug_res_conf_db = readGoldAssociations(input_fh, cluster_ids_rev)
        input_fh.close()
    
    input_fn = gold_dir + "/gold_assocs_A.txt"
    if not os.path.exists(input_fn):
        print("File does not exist:" + input_fn)
        drug_res_all_db = {}
    else:
        input_fh = open(input_fn)
        drug_res_all_db = readGoldAssociations(input_fh, cluster_ids_rev)
        input_fh.close()

    score_params_union = set([])    
    drug_names_map = {}
    drug_name_id = 0
    for drug_res_profile in drug_res_profiles:
        drug_name = drug_res_profile.drug_name

        for score_fn in os.listdir(scores_dir + "/" + drug_name):
            input_fn = scores_dir + "/" + drug_name  + "/" + score_fn
            if os.path.exists(input_fn): 
                score_tmp = score_fn[:-4]
                if score_tmp in scores_p:
                    score_params_union.add(score_tmp)
                if not drug_name in drug_names_map:
                    drug_names_map[drug_name] = drug_name_id
                    drug_name_id += 1

    print(','.join(drug_names_map.keys()))
    printGoldShort(drug_res_all_db, "all")
    printGoldShort(drug_res_conf_db, "hc")
    
    print("show0_save_associations.py: Finished.")
