import sys
import os
import copy
import random
from multiprocessing import Pool
from ete2 import Tree
from ete2 import PhyloTree
from ete2.treeview.main import TreeStyle, NodeStyle, TreeStyle
from ete2 import faces, layouts

sys.path.append("../../")

from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_res_io_utils import *
from drsoft.utils.gwamar_strains_io_utils import readStrains
from drsoft.utils.gwamar_utils import ensure_dir
from drsoft.utils.gwamar_tree_io_utils import readPhyloTrees
from drsoft.modelling.model_logic import *

def showTreeMutations(params):
    
    drug_name = params["DRUG_NAME"]
    
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    input_dir = parameters["DATASET_INPUT_DIR"]
    results_dir = parameters["RESULTS_DIR"]
    analysis_sel_dir = parameters["RESULTS_A_COMB_S_SEL_DIR"]
    
    input_fh = open(input_dir + "strains_ordered.txt")
    strains = readStrains(input_fh)
    input_fh.close()
    
    input_fh = open(input_dir + "res_profiles.txt")
    res_profiles_list = readResistanceProfiles(input_fh, load_int=True)
    input_fh.close()
    
    ensure_dir(input_dir + "/res_muts/")
    
    input_fh = open(results_dir + 'phylo_tree.txt')
    trees = readPhyloTrees(input_fh, strains)
    input_fh.close()
    
    tree = trees[0]

    res_profile = None
    for res_profile_tmp in res_profiles_list:
        if res_profile_tmp.drug_name == drug_name:
            res_profile = res_profile_tmp
    
    full_profile = res_profile.full_profile
    tree.setObservedResProfile(full_profile)
    tree.calcInnerNodesBU()
    calcSubleavesResCount(tree)
    calcSubnodesCount(tree)
    
    newtree = removeUnResLeaves(tree)
    newtree.calcInnerNodesBU()
    calcSubleavesResCount(newtree)
    calcSubnodesCount(newtree)
    calcDetParsResModel(newtree)
    
    tls = []
    
    tl = ""
    for node in newtree.inner_nodes_bu + newtree.leaves:
        if node.complete_res_state == "R" and node.parent.complete_res_state == "S":
            for leaf_node in node.getLeaves():
                tl += str(leaf_node.node_id) + ";"
            tl = tl[:-1] + "\t"
    tl = tl.strip()
    tls.append(tl)
    #tree_txt = newtree.root.toString(lengths = False, is_root=True, strains=strains)

    phylotree = Tree(newtree.toString(strains=strains) + ";")
    
    muts = {}
    mut_id = 1
    input_fn = analysis_sel_dir + "/" + drug_name + ".txt"
    input_fh = open(input_fn)
    for line in input_fh.readlines()[1:]:
        tokens = line.strip().split()
        if len(tokens) < 2:
            continue
        mut_profile = tokens[0]
        newtree.setObservedMutProfile(mut_profile)
        calcSubleavesMutCount(newtree)
        calcDetParsMutModel(newtree)

        tl = ""
        for node in newtree.inner_nodes_bu + newtree.leaves:
            if not node.complete_mut_state in ["0", "?"] and node.parent!=None and node.parent.complete_mut_state == "0":
                for leaf_node in node.getLeaves():
                    tl += str(leaf_node.node_id) + ";"
                tl = tl[:-1] + "\t"
        tl = tl.strip()
        tls.append(tl)
        mut_id += 1
        if mut_id >= 10: break
    input_fh.close()

    output_fn = results_dir + "/trees_data/" + drug_name + ".txt"
    output_fh = open(output_fn, "w")
    
    for tl in tls:
        output_fh.write(tl + "\n")
    output_fh.close()
    
    return None

    
if __name__ == '__main__':
    
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
        
    input_dir = parameters["DATASET_INPUT_DIR"]
    results_dir = parameters["RESULTS_DIR"]
    ensure_dir(results_dir + "/trees_data/")
        
    input_fh = open(input_dir + "res_profiles.txt")
    drug_names = readDrugNames(input_fh)
    input_fh.close()
    
    WORKERS = 1#int(parameters["W"])

    TASKS = []
    
    TASK = {}
    count = 0

    for drug_name in drug_names:
        TASK["DRUG_NAME"] = drug_name
        TASKS.append(TASK.copy())
                                                  #  scoreDrug(TASK)
    for TASK in TASKS:
        print(TASK)
    if WORKERS > 1:                               
        pool = Pool(processes=WORKERS)   
        for r in pool.imap(showTreeMutations, TASKS):
            print(r)
    else:
        for T in TASKS:
            showTreeMutations(T)

