import sys
import os
import copy
import random
from multiprocessing import Pool
from ete2 import Tree
from ete2 import PhyloTree, TreeNode, TreeFace
from ete2.treeview.main import TreeStyle, NodeStyle, TreeStyle
from ete2 import faces, layouts
from drsoft.utils.gwamar_progress_utils import GWADRIBProgress
from idlelib.ColorDelegator import prog

print(os.path.abspath(os.curdir))
sys.path.append("../../")

from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_res_io_utils import *
from drsoft.utils.gwamar_strains_io_utils import readStrains
from drsoft.utils.gwamar_utils import ensure_dir
from drsoft.utils.gwamar_tree_io_utils import readPhyloTrees
from drsoft.modelling.model_logic import *

mut_faces_map = {}
img_path = "./"

for i in range(1, 11, 1):
    mut_fn = os.path.abspath("../../resources/muts/m"+str(i)+".png")
    
    if not os.path.exists(mut_fn):
        print("not exists: ", mut_fn)
    mut_faces_map[i] = faces.ImgFace(mut_fn)
    
print(mut_faces_map)

resstyle = NodeStyle()
resstyle["fgcolor"] = "#ffbeb1"
resstyle["bgcolor"] = "#ffbeb1"

suscstyle = NodeStyle()
suscstyle["fgcolor"] = "lightgreen"
suscstyle["bgcolor"] = "lightgreen"

interstyle = NodeStyle()
interstyle["fgcolor"] = "yellow"
interstyle["bgcolor"] = "yellow"

nodes_face_map = {}

def mylayout(node):
    global nodes_face_map, mut_faces_map
    if node.is_leaf():
        descFace = faces.TextFace(node.name, fsize=10)
        descFace.margin_top = 10
        descFace.margin_bottom = 10
        descFace.border.margin = 1
        faces.add_face_to_node(descFace, node, column=1, aligned=True)
    
    node_hash = str(node)
    if not node_hash in nodes_face_map:
        pass
    else:
        for mut_id in sorted(nodes_face_map[node_hash]):
            faces.add_face_to_node(mut_faces_map[mut_id], node, 0)

    
def showTreeMutations(params):
    global nodes_face_map
    drug_name = params["DRUG_NAME"]
    
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    input_dir = parameters["DATASET_INPUT_DIR"]
    results_dir = parameters["RESULTS_DIR"]
    
    input_fh = open(input_dir + "strains_ordered.txt")
    strains = readStrains(input_fh)
    input_fh.close()
    
    input_fh = open(input_dir + "res_profiles.txt")
    res_profiles_list = readResistanceProfiles(input_fh, load_int=True)
    input_fh.close()
    
    ensure_dir(input_dir + "/res_muts/")
    
    input_fh = open(results_dir + 'phylo_tree.txt')
    trees = readPhyloTrees(input_fh, strains)
    input_fh.close()
    
    tree = trees[0]
    
    res_profile = None
    for res_profile_tmp in res_profiles_list:
        if res_profile_tmp.drug_name == drug_name:
            res_profile = res_profile_tmp
    
    full_profile = res_profile.full_profile
    tree.setObservedResProfile(full_profile)
    tree.calcInnerNodesBU()
    calcSubleavesResCount(tree)
    calcSubnodesCount(tree)
    
    newtree = removeUnResLeaves(tree)
    newtree.calcInnerNodesBU()
    calcSubleavesResCount(newtree)
    calcSubnodesCount(newtree)
    calcDetParsResModel(newtree)
   
    phylotree = Tree(newtree.toString(strains=strains) + ";")
    
    mut_id = 0
    input_fn = results_dir + "/trees_data/" + drug_name + ".txt"
    input_fh = open(input_fn)
    lines = input_fh.readlines()
    input_fh.close()

    line0 = lines[0]
    tokens = line0.strip().split()
    
    if len(tokens) < 1: exit()
    for mut_token in tokens:
        strain_id_tokens = mut_token.split(";")
        strain_ids = []
        for strain_index_str in strain_id_tokens:
            strain_ids.append(strains.getStrain(int(strain_index_str)))
        nodes_list = []
        for strain_id in strain_ids:
            nodes_list.append(phylotree.get_leaves_by_name(strain_id)[0])
        node0 = nodes_list[0]
        node = node0.get_common_ancestor(node0)
        for node1 in nodes_list[1:]:
            node = node.get_common_ancestor(node1)
        if phylotree.get_tree_root() == node:
            nodes_list = phylotree.get_leaves_by_name(strain_ids[0])
            for node in nodes_list:
                node.set_style(resstyle)
        else:
            node.set_style(resstyle)  
            #node.add_face(mut_faces_map[mut_id], 0)
    
    nodes_face_map = {}
    
    for line in lines[1:]:
        mut_id += 1
        if mut_id > 10:
            break
        tokens = line.strip().split()
        if len(tokens) < 1: continue
        for mut_token in tokens:
            strain_id_tokens = mut_token.split(";")
            strain_ids = []
            for strain_index_str in strain_id_tokens:
                strain_ids.append(strains.getStrain(int(strain_index_str)))
            nodes_list = []
            for strain_id in strain_ids:
                nodes_list.append(phylotree.get_leaves_by_name(strain_id)[0])
            node0 = nodes_list[0]
            node = node0.get_common_ancestor(node0)
            for node1 in nodes_list[1:]:
                node = node.get_common_ancestor(node1)
            hash_node = str(node)
            if not hash_node in nodes_face_map:
                nodes_face_map[hash_node] = set([])
            nodes_face_map[hash_node].add(mut_id)
        
    ts = TreeStyle()
    ts.show_leaf_name = False
    ts.layout_fn = mylayout
    ts.mode = "c"
    phylotree.render(results_dir +"/trees_muts_pdf/"+drug_name + ".pdf", tree_style=ts)
    #phylotree.show(tree_style=ts)
    
    return None

    
if __name__ == '__main__':
    
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
        
    input_dir = parameters["DATASET_INPUT_DIR"]
    results_dir = parameters["RESULTS_DIR"]
        
    ensure_dir(results_dir +"/trees_muts_pdf/")
    
    input_fh = open(input_dir + "res_profiles.txt")
    drug_names = readDrugNames(input_fh)
    input_fh.close()
    
    WORKERS = 1#int(parameters["W"])

    TASKS = []
    
    TASK = {}
    count = 0

    for drug_name in drug_names:
        TASK["DRUG_NAME"] = drug_name
        TASKS.append(TASK.copy())
                                                  #  scoreDrug(TASK)
    progress = GWADRIBProgress("PDF")
    progress.setJobsCount(len(TASKS))
    if WORKERS > 1:                               
        pool = Pool(processes=WORKERS)   
        for r in pool.imap(showTreeMutations, TASKS):
            progress.update(str(r))
    else:
        for T in TASKS:
            r = showTreeMutations(T)
            progress.update(str(r))

