import sys
import os
import copy
from ete2 import Tree
from ete2 import PhyloTree
from ete2.treeview.main import TreeStyle, NodeStyle, TreeStyle
from multiprocessing import Pool

sys.path.append("../../")

from drsoft.utils.gwamar_res_io_utils import readResistanceProfiles
from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_strains_io_utils import readStrains
from drsoft.utils.gwamar_utils import ensure_dir
from drsoft.utils.gwamar_tree_io_utils import readPhyloTrees
from drsoft.modelling.model_logic import *

nstyle = NodeStyle()
#nstyle["shape"] = "sphere"
nstyle["size"] = 10
nstyle["fgcolor"] = "black"

resstyle = NodeStyle()
#resstyle["shape"] = "sphere"
resstyle["size"] = 20
resstyle["fgcolor"] = "red"

suscstyle = NodeStyle()
#nstyle["shape"] = "sphere"
suscstyle["size"] = 20
suscstyle["fgcolor"] = "green"

interstyle = NodeStyle()
#nstyle["shape"] = "sphere"
interstyle["size"] = 20
interstyle["fgcolor"] = "yellow"


if __name__ == '__main__':
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    input_dir = parameters["DATASET_INPUT_DIR"]

    input_fh = open(input_dir + "strains_ordered.txt")
    strains = readStrains(input_fh)
    input_fh.close()

    input_fh = open(input_dir + "res_profiles.txt")
    res_profiles_list = readResistanceProfiles(input_fh, load_int=True)
    input_fh.close()
    
    ensure_dir(input_dir + "/res_trees/")
    
    input_fh = open(results_dir + 'phylo_tree.txt')
    trees = readPhyloTrees(input_fh, strains)
    input_fh.close()
   
    tree = trees[0]
   
    for res_profile in res_profiles_list:
        drug_name = res_profile.drug_name

        full_profile = res_profile.full_profile
        print(full_profile)
        tree.setObservedResProfile(full_profile)
        tree.calcInnerNodesBU()
        calcSubleavesResCount(tree)
        calcSubnodesCount(tree)
        
        newtree = removeUnResLeaves(tree)
        newtree.calcInnerNodesBU()
        calcSubleavesResCount(newtree)
        calcSubnodesCount(newtree)
        calcDetParsResModel(newtree)
        
        tree_txt = newtree.root.toString(lengths = False, is_root=True, strains=strains)
            
        genetree = Tree(tree_txt + ";")
    
        for node in genetree.traverse():
            node.set_style(nstyle)
    
        for i in range(len(full_profile)):
            strain_id = strains.getStrain(i)
            state = full_profile[i]
            if state == "R":
                for n in genetree.get_leaves_by_name(strain_id):
                    n.set_style(resstyle)
            elif state =="S":
                for n in genetree.get_leaves_by_name(strain_id):
                    n.set_style(suscstyle)
            elif state == "I":
                for n in genetree.get_leaves_by_name(strain_id):
                    n.set_style(interstyle)
                    

        genetree.render(input_dir + "/res_trees/"+drug_name+".png", w=600)

    
    
    