import sys
import os
import copy
from multiprocessing import Pool



sys.path.append("../../../../camber2/src/")
sys.path.append("../../")

from ressoft.utils.file_utils import ensure_dir
from soft.utils.camber_utils import *
from ressoft.utils.stats_utils import *
from ressoft.utils.generator import choose
from ressoft.utils.pred_utils import *
from ressoft.utils.input_utils import *
from ressoft.utils.names_utils import *
from ressoft.utils.output import *
from ressoft.modelling.model_logic import *


def printLatexTreeScoreNode(gene_profile, drug_res_profile, phylo_tree, node, strains, output_format_fh, strain_names=False):
    res_state = node.complete_res_state
    mut_state = node.complete_mut_state
    if res_state == 'R':
        res_color = "red"
    elif res_state == 'S':
        res_color = "green"
    else:
        res_color = "black"
    if mut_state == '1':
        mut_color = "orange"
    elif mut_state == '0':
        mut_color = "blue"
    else:
        mut_color = "black"
    
    if node != phylo_tree.root:
        parent_res_state = node.parent.complete_res_state
        parent_mut_state = node.parent.complete_mut_state
        tscore = stable(parent_res_state, res_state, parent_mut_state, mut_state, theta=0.5)
        if tscore > 0.0:
            output_format_fh.write("\edge[green] node{"+str(node.e_len)+"};\n")
        elif tscore < 0.0:
            output_format_fh.write("\edge[red] node{"+str(node.e_len)+"};\n")
        else:
            output_format_fh.write("\edge[black] node{"+str(node.e_len)+"};\n")
            
    output_format_fh.write("[."+str(node.tscore)+",")
    output_format_fh.write("\\textcolor{"+res_color+"}{"+str(res_state)+"},");
    output_format_fh.write("\\textcolor{"+mut_color+"}{"+str(mut_state)+"}");
    if len(node.children) == 0:
        if strain_names:
            output_format_fh.write("\\\\"+strains.getStrain(node.node_id).replace("_","-"));
        else:
            output_format_fh.write("\\\\"+str(node.node_id));
    else:
        pass
    output_format_fh.write("\n")
    
    for child_node in node.children:
        printLatexTreeScoreNode(gene_profile, drug_res_profile, phylo_tree, child_node, strains, output_format_fh, strain_names=True)
    
    output_format_fh.write("]\n");
        
    return None

def printLatexTreeScore(gene_profile, drug_res_profile, phylo_tree, strains, output_format_fh):
    output_format_fh.write(gene_profile.gene_id_text + "\n");
    output_format_fh.write("\\setlength{\\unitlength}{1mm}\n");
    output_format_fh.write("\\begin{tikzpicture}[level 1/.style={level distance=1.5cm}]\n");
    output_format_fh.write("\\tikzset{every tree node/.style={align=center,anchor=north}}\n");
    output_format_fh.write("\\tikzset{edge from parent/.append style={very thick}}\n");
    output_format_fh.write("\\Tree[\n");
    printLatexTreeScoreNode(gene_profile, drug_res_profile, phylo_tree, phylo_tree.root, strains, output_format_fh);
    output_format_fh.write("]\n");
    output_format_fh.write("\\end{tikzpicture}\n");
    return None

def drawTrees(drug_res_profile, drug_name, drug_targets, drug_res_genes, tree, all_mutations, gene_presence, strains, output_det_fh=None, output_format_fh=None, parameters=None):
    species = parameters["SPECIES"]
    details = parameters["DETAILS"]
    subset = parameters["SUBSET"]
    hide = int(parameters["HIDE"])
    beta = int(parameters["BETA"])
    pred = parameters["PRED"]
    dist = int(parameters["DIST"])
    exp_prefix = parameters["EXP_PREFIX"]
    grouping_type = parameters["GROUPING"] 
    m_type = parameters["TYPE"]
    input_dir = os.path.abspath("../../../data_input/"+species+"_res/")+"/"
    output_dir = os.path.abspath("../../../data_output/"+species+"_format/")+"/"

    calcRefSatesMutations(all_mutations, drug_res_profile)
    calcBinSatesMutations(all_mutations, drug_res_profile)
    con_mutations, neu_mutations, ess_mutations = categorizeMultiMutations(all_mutations, drug_res_profile)
    imp_mutations = con_mutations + neu_mutations + ess_mutations
    gene_profiles = geneMutations(imp_mutations)
    con_gene_profiles = geneMutations(con_mutations)
    
    tree.setObservedResProfile(drug_res_profile.full_profile)

    calcSubleavesResCount(tree)
    calcSubnodesCount(tree)
    calcDetParsResModel(tree)
    
    diffTime("Classification: "+drug_name)
    calcMutCoverSupports(imp_mutations, drug_res_profile);
    diffTime("Coverage support: "+drug_name)
    gene_profiles = groupGeneMutations(gene_profiles, dist, con_gene_profiles, grouping_type)
    diffTime("Grouping: "+drug_name)
    computeGeneProfiles(gene_profiles)
    diffTime("Gene profiles: "+drug_name)
    computeGeneProfilesEss(gene_profiles)
    
    diffTime("Coverage support: "+drug_name)
    if beta == -1:
        beta_str = str("%.2f" % float(len(drug_res_profile.getResistant())/float(len(drug_res_profile.getSusceptible()))))
        beta = float(beta_str)
        print(drug_name, "NEW BETA: ", beta)
    
    diffTime("Gene profiles essentiality: "+drug_name)
    calcGenesCoverSupports(gene_profiles, drug_res_profile, beta)    
    tree.calcContributions(drug_res_profile, beta)  
    calcGenesWeightedSupports(gene_profiles, drug_res_profile, tree, beta)
    diffTime("Weighted support: "+drug_name)
    calcOddsRatio(gene_profiles, drug_res_profile)
    diffTime("Odds ratio: "+drug_name)
    
    newtree = removeUnResLeaves(tree)
    newtree.setObservedResProfile(drug_res_profile.full_profile)
    
    
    print("TREE", newtree.leaves_count, newtree.nodes_count)
    for i in newtree.leaf_ids:
        children_ids = ""
        for child in newtree.nodes[i].children:
            children_ids += str(child.node_id) + ","
        if newtree.nodes[i].parent == None:
            prant_id = "null"
        else:
            prant_id = str(newtree.nodes[i].parent.node_id) 
        print("TREE", i, "-->", children_ids, " | ", prant_id)

    calcSubleavesResCount(newtree)
    calcSubnodesCount(newtree)
    calcDetParsResModel(newtree)
    
    gene_profiles_sorted = sorted(gene_profiles, key=lambda gp:-gp.wsupport)
    thr = list(gene_profiles_sorted)[4].wsupport

    count = 0
    tscore_res_map = {}
    wsupport_res_map = {}

    for gene_profile in gene_profiles_sorted:
        count += 1
        if gene_profile.wsupport < thr:
            continue
        key = gene_profile.getFullBinKey()
        if not key in tscore_res_map:
            newtree.setObservedMutProfile(gene_profile.bin_profile)
      
            calcSubleavesMutCount(newtree)
            calcDetParsMutModel(newtree)
            gene_profile.tscore = calcTransitionScore(newtree, 0.5)
      
            tscore_res_map[key] = gene_profile.tscore
            wsupport_res_map[key] = 0.0  
        else:
            gene_profile.tscore = tscore_res_map[key]
            gene_profile.exp_wsupport = wsupport_res_map[key]

        output_format_fn = output_dir +"/latex/"+str(exp_prefix)+"-"+str(hide)+"-"+str(dist)+"-"+grouping_type+"-"+details+"-"+subset+"-"+m_type+"-"+pred+"-"+str(beta)+"-"+drug_name+"_"+str(count)+".tex"
        output_format_fh = open(output_format_fn, "w")
        printLatexTreeScore(gene_profile, drug_res_profile, newtree, strains, output_format_fh)
        output_format_fh.close();
    return None


#     output_format_fh.close()

def scoreDrug(parameters):
 #   print(drug_name)
 #   parameters = readParameters(sys.argv)
    species = parameters["SPECIES"] 
    subset = parameters["SUBSET"]
    m_type = (parameters["TYPE"])
    pred = (parameters["PRED"])
    drug_name = parameters["DRUG_NAME"]

    phase = 0
    input_dir = os.path.abspath("../../../data_input/"+species+"_res/")+"/"
    output_dir = os.path.abspath("../../../data_output/"+species+"_out/")+"/"
    format_dir = os.path.abspath("../../../data_output/"+species+"_format/")+"/"
    
    ensure_dir(format_dir)
    ensure_dir(format_dir +"/latex/")
    ensure_dir(output_dir)
    
    input_fh = open(input_dir + "strains-all.txt")
    strains = readStrains(input_fh)
    input_fh.close()
    
    if pred == "B":
        input_fn = input_dir + "res_profiles_ret.txt"
    else:
        input_fn = input_dir + "pred_res_profiles.txt"
    input_fh = open(input_fn)
    res_profiles_list = readResistanceProfiles(input_fh, strains)
    input_fh.close()
    
    res_profiles = {}
    for res_profile in res_profiles_list:
        res_profiles[res_profile.drug_name] = res_profile
    
    cluster_fh = open(input_dir + "conn_comp-det-"+str(phase)+".txt")
    clusters, id_cluster_map = readClusters(cluster_fh, strains)
    cluster_fh.close()
    
    anns_fn = input_dir+"names_anns.txt"
    if os.path.exists(anns_fn):
        anns_fh = open(anns_fn)
        names_mapping = readNamesMapping(anns_fh)
        anns_fh.close()
    else:
        names_mapping = None
    
    strains_ord_fh = open(input_dir + "names-strains-ord.txt")
    strains_ord = readStrainNamesOrd(strains_ord_fh)
    strains_ord_fh.close()
    
    input_fh = open(input_dir + "drug_targets.txt")
    drug_targets = loadDrugTargets(input_fh, clusters, id_cluster_map, strains_ord)
    input_fh.close()
    
    input_fh = open(input_dir + "drug_res_genes.txt")
    drug_res_genes = loadDrugResGenes(input_fh, clusters, id_cluster_map, strains_ord)
    input_fh.close()

    input_fh = open(results_dir + 'phylo_tree.txt')
    tree = readPhyloTree(input_fh, strains)
    input_fh.close()
    
    print("TREE", tree.leaves_count, tree.nodes_count)
    for i in range(tree.nodes_count):
        children_ids = ""
        for child in tree.nodes[i].children:
            children_ids += str(child.node_id) + ","
        if tree.nodes[i].parent == None:
            prant_id = "null"
        else:
            prant_id = str(tree.nodes[i].parent.node_id) 
        print("TREE", i, "-->", children_ids, " | ", prant_id)
    
    if subset == "Y":
        input_fh = open(input_dir + 'subset_imp_genes_neigh.txt')
        gene_subset = readGeneSubset(input_fh, clusters, id_cluster_map, strains_ord)
        input_fh.close()
    else:
        gene_subset = None
        
    input_fh = open(input_dir + 'man-all-mutations-'+str(phase)+'.txt')
    all_mutations, gene_presence = readMultiMutations(input_fh, strains, clusters=clusters, id_cluster_map=id_cluster_map, strains_ord=strains_ord, names_mapping=names_mapping, gene_subset=gene_subset, m_type=m_type)
    input_fh.close()
    
    print("READ")
    sys.stdout.flush()
    
    imp_mutations = filterMutations(all_mutations, strains)
    
    
    diffTime("FILTERED")
    drug_res_profile = copy.deepcopy(res_profiles[drug_name])
    drawTrees(drug_res_profile, drug_name, drug_targets, drug_res_genes, tree, imp_mutations, gene_presence, strains, parameters=parameters)
    diffTime("EVALUATED")
    
    return drug_name
    
if __name__ == '__main__':
    
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    
    if "W" in parameters:
        WORKERS = int(parameters["W"])
    else:
        WORKERS = 4
    #species = "sta_plas_184"
    #species = "mtb68"
    if "SPECIES" in parameters and parameters["SPECIES"] != "mtb":
        species = parameters["SPECIES"]
    else:
        species = "sta_plas_184"
    
    
    input_dir = os.path.abspath("../../../data_input/"+species+"_res/")+"/"
    output_dir = os.path.abspath("../../../data_output/"+species+"_out/")+"/"
    format_dir = os.path.abspath("../../../data_output/"+species+"_format/")+"/"
    
    ensure_dir(output_dir)
    ensure_dir(format_dir)
    #scripts_dir = "/research/wongls-group/wozniak/scripts_pred/"
    #scripts_dir = "../../"+species+"-scripts_pred/"
    #ensure_dir(scripts_dir)
    
    cur_dir = os.path.abspath(os.curdir) +"/"
    
    hide_params = set(["0"])
    dist_params = set(["0"])
    pred_params = set(["A"])
    beta_params = set(["-1"])
    grouping_params = set(["B"])
    details_params = set(["S"])
    subset_params = set(["N"])
    type_params = set(["S"])
    exp_prefix = "FINAL3"
    
    input_fh = open(input_dir + "res_profiles_ret.txt")
    drug_names = readDrugNames(input_fh)
    input_fh.close()
    print(drug_names)
        
    TASKS = []
    
    TASK = {}
    count = 0
    for beta_param in beta_params:
        for pred_param in pred_params:
            for type_param in type_params:
                for hide_param in hide_params:
                    for dist_param in dist_params:
                        for grouping_param in grouping_params:
                            for details_param in details_params:
                                for subset_param in subset_params:
                                    for drug_name in drug_names:
                                        if drug_name != "Tetracycline":
                                            continue
                                        TASK["SPECIES"] = species
                                        TASK["PHASE"] = str(0)
                                        TASK["DETAILS"] = str(details_param)
                                        TASK["HIDE"] = str(hide_param)
                                        TASK["SUBSET"] = str(subset_param)
                                        TASK["DIST"] = str(dist_param)
                                        TASK["GROUPING"] = str(grouping_param)
                                        TASK["EXP_PREFIX"] = exp_prefix 
                                        TASK["TYPE"] = type_param
                                        TASK["PRED"] = pred_param
                                        TASK["BETA"] = beta_param 
                                        TASK["DRUG_NAME"] = drug_name
                                   #     TASK["TOP"] = parameters["TOP"]
                                        TASKS.append(TASK.copy())
  #                                      scoreDrug(TASK)

    for TASK in TASKS:
        print(TASK)
                                        
    pool = Pool(processes=WORKERS)   
    for r in pool.imap(scoreDrug, TASKS):
        print(r)
        
        

