import sys
import os
import copy
from multiprocessing import Pool

sys.path.append("../../")

from drsoft.utils.gwamar_res_io_utils import readResistanceProfiles
from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_strains_io_utils import readStrains
from drsoft.utils.gwamar_utils import ensure_dir
from drsoft.utils.gwamar_tree_io_utils import readPhyloTrees
from drsoft.modelling.model_logic import *


if __name__ == '__main__':
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    input_dir = parameters["DATASET_INPUT_DIR"]
    results_dir = parameters["RESULTS_DIR"]

    input_fh = open(input_dir + "strains_ordered.txt")
    strains = readStrains(input_fh)
    input_fh.close()

    strains_list = strains.allStrains()

    input_fh = open(input_dir + "res_profiles.txt")
    res_profiles_list = readResistanceProfiles(input_fh, load_int=True)
    input_fh.close()
    
    ensure_dir(input_dir + "/res_trees_dots/")
    ensure_dir(input_dir + "/res_trees_pdfs/")

    input_fh = open(results_dir + 'tree.txt')
    trees = readPhyloTrees(input_fh, strains_list)
    input_fh.close()
   
    tree = trees[0]
   
    for res_profile in res_profiles_list:
        drug_name = res_profile.drug_name

        tls = []
        tls.append("digraph "+ "G" + " {\n")
        full_profile = res_profile.full_profile
        print(full_profile)
        tree.setObservedResProfile(full_profile)
        tree.calcInnerNodesBU()
        calcSubleavesResCount(tree)
        calcSubnodesCount(tree)
        
        newtree = removeUnResLeaves(tree)
        newtree.calcInnerNodesBU()
        calcSubleavesResCount(newtree)
        calcSubnodesCount(newtree)
        calcDetParsResModel(newtree)

        for node in newtree.nodes.values():
            if not node.is_leaf():
                if node.complete_res_state == "R": color = "red"
                elif node.complete_res_state == "I": color = "yellow"
                elif node.complete_res_state == "S": color = "green"
                else: color = "white"
                bgcolor = "white"                
            else:
                if node.complete_res_state == "R": bgcolor = "red"
                elif node.complete_res_state == "I": bgcolor = "yellow"
                elif node.complete_res_state == "S": bgcolor = "green"
                else: bgcolor = "white"
                color = "black"

            pdf_label_tmp = str(node.label).replace("_", "").replace("-","")
            if pdf_label_tmp[0] in ["0","1","2","3","4","5","6","7","8","9"]:
                pdf_label = "n"+pdf_label_tmp
            else: pdf_label = pdf_label_tmp            
            tls.append(str(node.node_id) + " [label="+pdf_label+",style=filled,color="+color+",fillcolor=" + bgcolor + "];\n")
           # pdf_label_tmp = str(strains_list[node.label]).replace("_", "").replace("-","")
           # if pdf_label_tmp[0] in ["0","1","2","3","4","5","6","7","8","9"]:
           #     pdf_label = "n"+pdf_label_tmp
           # else: pdf_label = pdf_label_tmp
        #    tls.append(str(node.node_id) + " [label="+pdf_label+",color=" + color + "];\n")

        for node in newtree.inner_nodes_bu: 
            for node_c in node.children:
                tls.append(str(node.node_id) + "->" + str(node_c.node_id) + ";\n")

        tls.append("}\n")
                    
        output_dot_fn = input_dir + "/res_trees_dots/"+drug_name+".dot"
        output_pdf_fn = os.path.relpath(input_dir + "/res_trees_pdfs/"+drug_name + ".pdf")

        output_fh = open(output_dot_fn, "w")
        for tl in tls:
            output_fh.write(tl)
        output_fh.close()

        os.system("dot -Tpdf " + output_dot_fn + " -o " + output_pdf_fn)
