import sys
import os
import copy
import shutil
from multiprocessing import Pool

sys.path.append("../../")

from drsoft.utils.gwamar_res_io_utils import readResistanceProfiles
from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_strains_io_utils import readStrains
from drsoft.utils.gwamar_utils import ensure_dir
from drsoft.utils.gwamar_bin_utils import *
from drsoft.utils.gwamar_tree_io_utils import *
from drsoft.modelling.model_logic import *


color_map = {
"1": "blue",
"2": "green",
"3": "orange",
"4": "pink",
"5": "brown"}

def addLegend(): 
    tls = []
    tls.append("subgraph Legend {\n")
    tls.append("k1c [label=\"Susceptible strain\",shape=plaintext,fontsize=20];\n")
    tls.append("k2c [label=\"Intermediate strain\",shape=plaintext,fontsize=20];\n")
    tls.append("k3c [label=\"Resistant strain\",shape=plaintext,fontsize=20];\n")
    tls.append("m1c [label=\"No mutation edge\",shape=plaintext,fontsize=20];\n")
    tls.append("m2c [label=\"Mutation edge\",shape=plaintext,fontsize=20];\n")
    tls.append("k1 [label=\"strain1\",fontsize=16,style=filled,color=black,fillcolor=green];\n")
    tls.append("k2 [label=\"strain2\",fontsize=16,style=filled,color=black,fillcolor=yellow];\n")
    tls.append("k3 [label=\"strain3\",fontsize=16,style=filled,color=black,fillcolor=red];\n")
    tls.append("m1x [label=\"strain4\",fontsize=16,style=filled,color=green,fillcolor=white];\n")
    tls.append("m1y [label=\"strain5\",fontsize=16,style=filled,color=green,fillcolor=white];\n")
    tls.append("m2x [label=\"strain6\",fontsize=16,style=filled,color=red,fillcolor=white];\n")
    tls.append("m2y [label=\"strain7\",fontsize=16,style=filled,color=red,fillcolor=white];\n")
        
    tls.append("m1x->m1y[penwidth=1,color=black];\n")
    tls.append("m2x->m2y[penwidth=4,color=blue];\n")
    tls.append("k1c->k2c->k3c->m1c->m2c[style=invis];\n")
    tls.append("k1->k2->k3->m1x->m2x[style=invis];\n")
    tls.append("{ rank=same;m1x m1y}\n")
    tls.append("{ rank=same;m2x m2y}\n")
    tls.append("{ rank=same;k1c k1}\n")
    tls.append("}\n");


    return tls


if __name__ == '__main__':
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    input_dir = parameters["DATASET_INPUT_DIR"]
    results_dir = parameters["RESULTS_DIR"]
    scores_dir = parameters["RESULTS_SCORES_DIR"]

    score = "mi"

    input_fh = open(input_dir + "strains_ordered.txt")
    strains = readStrains(input_fh)
    input_fh.close()

    strains_list = strains.allStrains()

    input_fh = open(input_dir + "res_profiles.txt")
    res_profiles_list = readResistanceProfiles(input_fh, load_int=True)
    input_fh.close()
    
    ensure_dir(results_dir + "/res_mut_trees_dots/")
    ensure_dir(results_dir + "/res_mut_trees_pdfs/")

    input_fh = open(results_dir + 'tree.txt')
    trees = readPhyloTrees(input_fh, strains_list)
    input_fh.close()
   
    tree = trees[0]


    input_fn = input_dir + '/bin_profiles_' + parameters["MT"] + '.txt'
    if not os.path.exists(input_fn):
        sys.exit(-1)

    input_fh = open(input_fn)
    bin_profiles = readBinProfiles(input_fh, strains.count(), gen_bin_profiles=True)
    input_fh.close()

    input_fh = open(input_dir + '/bin_profiles_' + parameters["MT"] + '_det.txt')
    mut_details_map = readBinProfileDetails(input_fh, strains.count(), parameters["MT"])
    input_fh.close()

    for res_profile in res_profiles_list:
        drug_name = res_profile.drug_name
        try:
            util.rmtree(results_dir + "/res_mut_trees_dots/" + drug_name + "/")
            util.rmtree(results_dir + "/res_mut_trees_pdfs/" + drug_name + "/")
        except:
            pass

        ensure_dir(results_dir + "/res_mut_trees_dots/" + drug_name + "/")
        ensure_dir(results_dir + "/res_mut_trees_pdfs/" + drug_name + "/")

        
        input_fn = scores_dir + drug_name + "/" + score + ".txt"
        if not os.path.exists(input_fn) or os.path.getsize(input_fn) < 10:
            print("NOT EXIST: ", input_fn)
            continue
        
        input_fh = open(input_fn)
        bin_profiles_sorted = readBinProfilesSorted(input_fh, False)
        input_fh.close()
        
        n = len(bin_profiles_sorted)

        bin_profiles_set = set([])
        ranks_map = {}
        for i in range(min(n,10)):
            (bin_profile_id, score_tmp) = bin_profiles_sorted[i]
            bin_profiles_set.add(bin_profile_id)
            ranks_map[bin_profile_id] = i + 1

        full_profile = res_profile.full_profile

        tree.setObservedResProfile(full_profile)
        tree.calcInnerNodesBU()
        calcSubleavesResCount(tree)
        calcSubnodesCount(tree)
        
        newtree = removeUnResLeaves(tree)
        newtree.calcInnerNodesBU()
        calcSubleavesResCount(newtree)
        calcSubnodesCount(newtree)
        calcDetParsResModel(newtree)

        for bin_profile_id in bin_profiles_set:
            bin_profile = bin_profiles[bin_profile_id]
            details = mut_details_map[bin_profile_id]

            print(drug_name, ''.join(bin_profile), bin_profile_id)

            newtree.setObservedMutProfile(bin_profile)
            calcSubleavesMutCount(newtree)
            calcDetParsMutModel(newtree)

            tls = []
            tls.append("digraph "+ "G" + " {\n")
            tls.append("fontsize=30\n");
            tls.append("label=\"" + details.replace(";", "  ").replace("_", "-")[:30] + "\"\n");

            for node in newtree.nodes.values():
                if not node.is_leaf():
                    if node.complete_res_state == "R": color = "red"
                    elif node.complete_res_state == "I": color = "yellow"
                    elif node.complete_res_state == "S": color = "green"
                    else: color = "white"
                    bgcolor = "white"                
                else:
                    if node.complete_res_state == "R": bgcolor = "red"
                    elif node.complete_res_state == "I": bgcolor = "yellow"
                    elif node.complete_res_state == "S": bgcolor = "green"
                    else: bgcolor = "white"
                    color = "black"

                pdf_label_tmp = str(node.label).replace("_", "").replace("-","")
                if pdf_label_tmp[0] in ["0","1","2","3","4","5","6","7","8","9"]:
                    pdf_label = "n"+pdf_label_tmp
                else: pdf_label = pdf_label_tmp            
                tls.append(str(node.node_id) + " [label="+pdf_label+",style=filled,color="+color+",fillcolor=" + bgcolor + "];\n")

            for node in newtree.inner_nodes_bu: 
                for node_c in node.children:
                    if node_c.complete_mut_state == "?":
                        ecolor = "grey"
                        pencolor = "1"
                    elif node_c.complete_mut_state == "0":
                        ecolor = "black"
                        pencolor = "1"
                    else:
                        ecolor = color_map.get(node_c.complete_mut_state, "purple")
                        pencolor = "4"
                    tls.append(str(node.node_id) + "->" + str(node_c.node_id) + " [penwidth="+pencolor+",color="+ecolor+"];\n")

            tls += addLegend()
            tls.append("}\n")
                        
            output_dot_fn = results_dir + "/res_mut_trees_dots/" + drug_name + "/"+str(ranks_map[bin_profile_id])+".dot"
            output_pdf_fn = os.path.relpath(results_dir + "/res_mut_trees_pdfs/"+drug_name + "/"+str(ranks_map[bin_profile_id]) +".pdf")

            output_fh = open(output_dot_fn, "w")
            for tl in tls:
                output_fh.write(tl)
            output_fh.close()

            os.system("dot -Tpdf " + output_dot_fn + " -o " + output_pdf_fn)
