import sys
import os
import shutil
import copy
from multiprocessing import Pool
import multiprocessing

sys.path.append("../../")

from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_ann_utils import readClusterGeneIDs
from drsoft.utils.gwamar_muts_utils import *
from drsoft.utils.gwamar_progress_utils import GWAMARProgress
from drsoft.utils.gwamar_res_utils import *
from drsoft.utils.gwamar_muts_io_utils import *
from drsoft.utils.gwamar_tree_io_utils import *
from drsoft.utils.gwamar_utils import ensure_dir
from drsoft.utils.gwamar_res_io_utils import *
from drsoft.utils.gwamar_strains_io_utils import readStrains


if __name__ == '__main__':
    print("p1_prepare_experiments.py: prepares folders to store results. Start.")
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    tree_ids=["PHYML", "RAXML", "", "PHYLIP"]
    tree_ids.append(parameters.get("TREE_ID", ""))

    WORKERS = min(int(parameters["W"]), multiprocessing.cpu_count())
    
    input_dir = parameters["DATASET_INPUT_DIR"]

    input_fh = open(input_dir + "/strains_ordered.txt")
    strains = readStrains(input_fh)
    input_fh.close()
    
    strains_list = strains.allStrains()
    
    input_fh = open(input_dir + "/res_profiles.txt")
    res_profiles = readResistanceProfiles(input_fh, load_int=True, strains_list=strains_list)
    input_fh.close()
    
    output_dir = parameters["DATASET_DIR"] + "/"+parameters["EP"]+"P/"
    ensure_dir(output_dir)

    output_fh = open(output_dir + "res_profiles.txt", "w")
    saveResistanceProfiles(output_fh, res_profiles, strains_list, compress=True)
    output_fh.close()

    for tree_id in tree_ids:
        output_dir = parameters["DATASET_DIR"] + "/"+parameters["EP"]+"P/"
        
        input_fn = input_dir + "tree"+tree_id+".txt"
        if not os.path.exists(input_fn):
            continue

        input_fh = open(input_fn)
        tree = readPhyloTrees(input_fh, strains_list)
        input_fh.close()

        output_tree_txt = tree[0].toString(lengths=False, strains=strains)
        output_fh = open(output_dir + "/tree"+tree_id+".txt", "w")
        output_fh.write(output_tree_txt+ ";\n")
        output_fh.close()
        
        output_dir = parameters["DATASET_DIR"] + "/"+parameters["EP"]+"G/"
        ensure_dir(output_dir)
        
        output_fh = open(output_dir + "res_profiles.txt", "w")
        saveResistanceProfiles(output_fh, res_profiles, strains_list, compress=True)
        output_fh.close()

        output_tree_txt = tree[0].toString(lengths=False, strains=strains)
        output_fh = open(output_dir + "/tree"+tree_id+".txt", "w")
        output_fh.write(output_tree_txt+ ";\n")
        output_fh.close()

    print("p1_prepare_experiments.py: Finished.")
    