import sys
import os
import copy
from multiprocessing import Pool
import multiprocessing

sys.path.append("../../")

from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_ann_utils import *
from drsoft.utils.gwamar_muts_utils import *
from drsoft.utils.gwamar_res_utils import *
from drsoft.utils.gwamar_strains_io_utils import *
from drsoft.utils.gwamar_muts_io_utils import *
from drsoft.utils.gwamar_res_io_utils import *
from drsoft.utils.gwamar_progress_utils import *
from drsoft.utils.gwamar_bin_utils import *
from drsoft.utils.gwamar_utils import ensure_dir


profile_clusters = {
"Fluoroquinolones":["Gatifloxacin","Ofloxacin","Moxifloxacin","Levofloxacin","Ciprofloxacin"],
"Aminoglycosides":["Amikacin","Kanamycin","Capreomycin"],
"Rifampicin":["Rifampicin","Rifabutin"]
 }  

def combineResistanceProfiles(res_profiles):
    res_profiles_comb = []

    combined = set([])
    res_profiles_map = {}
    for drp in res_profiles: res_profiles_map[drp.drug_name] = drp

 #   print(res_profiles_map.keys())
    
    for cluster_id in profile_clusters:
        cluster_drp = ResistanceProfile(cluster_id, count = k)

        for drug_name in profile_clusters[cluster_id]:
            drp = res_profiles_map.get(drug_name, None)
            if drp!= None:
                combined.add(drp.drug_name)

        for i in range(k):
            res_count = 0
            int_count = 0
            susc_count = 0
            for drug_name in profile_clusters[cluster_id]:
                drp = res_profiles_map.get(drug_name, None)

                if drp != None:

                    if drp.full_profile[i] == "R": res_count += 1
                    elif drp.full_profile[i] == "I": int_count += 1
                    elif drp.full_profile[i] == "S": susc_count += 1

            if res_count > 0 and susc_count >0:
              #  print("conflict", cluster_id, profile_clusters[cluster_id], i, strains_list[i], susc_count, int_count, res_count)
                cluster_drp.changeState(i, "?")
            elif res_count >= 1 and susc_count == 0:
                cluster_drp.changeState(i, "R")
            elif res_count == 0 and susc_count >= 1:
                cluster_drp.changeState(i, "S")
            elif res_count == 0 and susc_count == 0 and int_count >=1:
                cluster_drp.changeState(i, "I")
            else:
                cluster_drp.changeState(i, "?")
#        print(''.join(cluster_drp.full_profile), cluster_drp.drug_name)
        res_profiles_comb.append(cluster_drp)

#    print(combined)

    # for drp in res_profiles_comb:
    #     drp.computeResSets()
    #     print(drp.drug_name, len(drp.getKnown()))

    for drp in res_profiles:
   #     if not drp.drug_name in combined:
        res_profiles_comb.append(drp)
    return res_profiles_comb

if __name__ == '__main__':
    print("p5_cmp_resistance_profiles.py: preparation of resistnce profiles for comparison of different association scores. Start.")

    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    m_type = parameters["MT"]
    input_dir = parameters["DATASET_INPUT_DIR"]
    results_dir = parameters["RESULTS_DIR"]

    sel_profiles = parameters["CMP_PROFILES"].split(",")

    input_fh = open(input_dir + "/strains_ordered.txt")
    strains = readStrains(input_fh)
    input_fh.close()
    
    strains_list = strains.allStrains()
    k = len(strains_list)

    input_fn = input_dir + "/res_profiles_ret.txt"
    if not os.path.exists(input_fn):
        input_fn =  input_dir + "/res_profiles.txt"
        if not os.path.exists(input_fn):
            print("File " + input_fn + " does not exist.")
            sys.exit()
    output_fn = input_dir + "/res_profiles_cmp.txt"
    if os.path.exists(output_fn):
        print("Output file: " + output_fn + " already exist.")
        print("In order to use them replace manually this file with: ")
        print(results_dir + "/res_profiles_cmp.txt")
    else:
        input_fh = open(input_fn)
        res_profiles = readResistanceProfiles(input_fh, strains_list=strains.allStrains(), load_int=True)
        input_fh.close()

        res_profiles_comb = combineResistanceProfiles(res_profiles)
        
        output_fh = open(output_fn, "w")
        saveResistanceProfiles(output_fh, res_profiles_comb, strains_list, compress=True, sel_profiles=sel_profiles)
        output_fh.close()

        print("Profiles used for comparison of the association scores are generated in the file: ")
        print(output_fn)
        print("In order to use them replace manually this file with: ")
        print(results_dir + "/res_profiles_cmp.txt")
    
    print("p5_cmp_resistance_profiles.py. Finished")
