import sys
import os
import copy
import random
from multiprocessing import Pool
import multiprocessing

sys.path.append("../../")

from drsoft.utils.gwamar_params_utils import *
from drsoft.utils.gwamar_tree_io_utils import readPhyloTrees
from drsoft.utils.gwamar_bin_utils import *
from drsoft.utils.gwamar_utils import ensure_dir
from drsoft.utils.gwamar_progress_utils import GWAMARProgress
from drsoft.utils.gwamar_strains_io_utils import readStrains
from drsoft.utils.gwamar_res_io_utils import readResistanceProfiles
from drsoft.utils.gwamar_scoring import revOrder


def scoreParamsSet(params):
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    score = params["SCORE"]

    rev_order = revOrder(score)

    input_dir = parameters["DATASET_INPUT_DIR"]
    scores_dir = parameters["RESULTS_SCORES_DIR"]
    ranks_dir = parameters["RESULTS_RANKS_DIR"]
    results_dir = parameters["RESULTS_DIR"]

    input_fh = open(input_dir + "/strains_ordered.txt")
    strains = readStrains(input_fh)
    input_fh.close()

    input_fh = open(results_dir + "/res_profiles.txt")
    drug_res_profiles = readResistanceProfiles(input_fh, load_int=True)
    input_fh.close()
    
    input_fh = open(results_dir + '/bin_profiles_' + parameters["MT"] + '_count.txt')
    bin_profile_count = readBinProfileCounts(input_fh)
    input_fh.close()
    
    for drug_res_profile in drug_res_profiles:
        drug_name = drug_res_profile.drug_name
        tls = []
       # print(bin_profiles_res_rel)
        input_fn = scores_dir + drug_name + "/" + score + ".txt"
        if not os.path.exists(input_fn) or os.path.getsize(input_fn) < 10:
            print("NOT EXIST: ", input_fn)
            continue
        
        ensure_dir(ranks_dir + drug_name)
                
        input_fh = open(input_fn)
        bin_profiles_sorted = readBinProfilesSorted(input_fh, rev_order)
        input_fh.close()
        
        n = len(bin_profiles_sorted)
        bin_profiles_ranks = computeRankings(bin_profiles_sorted, bin_profile_count)

        for i in range(n):
            bin_profile_id, bin_profile_score = bin_profiles_sorted[i]
            ranks_list = list(bin_profiles_ranks[bin_profile_id])

            tl = "\t".join(map(str, [bin_profile_id] + ranks_list)) + "\n"
            tls.append(tl)
    
        output_fn = ranks_dir + str(drug_name) + "/" + str(score) + ".txt"    
        output_fh = open(output_fn, "w")
        for tl in tls:
            output_fh.write(tl)
        output_fh.close()
               
    return score

if __name__ == '__main__':
    print("sc4_save_perm_rankings.py. Rankings for the permutation pvalue scores.")
    parameters = overwriteParameters(sys.argv)
    parameters = readParameters()
    
    WORKERS = min(int(parameters["W"]), multiprocessing.cpu_count())
    
    score_params = parameters["SCORES"].split(",")
    perm_rep = parameters["REPS"]
    
    TASKS = []
    TASK = {}

    for score_param in score_params:
        TASK["SCORE"] = "pv-" + str(perm_rep) + "-" + score_param

        TASKS.append(TASK.copy())

        
    progress = GWAMARProgress("Rankings computed")
    progress.setJobsCount(len(TASKS))
      
    WORKERS = min(WORKERS, len(TASKS))

    if WORKERS > 1:
        pool = Pool(processes=WORKERS)   
        for r in pool.imap(scoreParamsSet, TASKS):
            progress.update(str(r))
    else:
        for T in TASKS:
            r = scoreParamsSet(T)
            progress.update(str(r))
                  
    print("sc4_save_perm_rankings.py. Finished.")
