import sys
import os
import copy
sys.path.append("../../")
from ressoft.utils.pvalue_utlis import hyperTestPvalue
from multiprocessing import Pool



from ressoft.utils.stats_utils import *
from ressoft.utils.generator import choose
from ressoft.utils.pred_utils import *
from ressoft.utils.input_utils import *
from ressoft.utils.names_utils import *
from ressoft.utils.output import *
from ressoft.utils.file_utils import *

def readParameters(param_tab):
    parameters = {}
    for param_el in param_tab:
        tokens = param_el.split("=")
        if len(tokens)==2:
            key = tokens[0]
            value = tokens[1]
            parameters[key] = value
        
    return parameters

parameters = readParameters(sys.argv)

print(parameters)

 
def filterTopGeneProfiles(gene_profiles, n):
    subset = set([])
    
    for gene_profile in gene_profiles:
        if gene_profile.norm_wsupport > 0.0:
            subset.add(gene_profile)
    subset_sorted = sorted(subset, key=lambda gp:(-gp.wsupport, -gp.cover_support))    
    return subset_sorted[:n]
            
def filterGeneProfilesByPvalue(gene_profiles, drug_res_genes, target_genes):
    subset = set([])
    for gene_profile in gene_profiles[:10]:
        subset.add(gene_profile)
    subset_sorted = sorted(subset, key=lambda gp:(gp.pvalue_wsupport, -gp.cover_support))
    return subset_sorted[:10]
            
    return None


def loadGeneFunctions(input_fh):
    gene_ids = {}
    for line in input_fh.readlines():
        if not line:
            continue
        line = line.strip()
        tokens = line.split("\t")
        if len(tokens) <= 1:
            continue
        gene_id = tokens[0]
        function_id = tokens[3].lower()
        function_id = function_id.replace("_", "\_")
        if not gene_id in gene_ids:
            gene_ids[gene_id] = function_id
    return gene_ids

def decorateScors(parameters):
    species = parameters["SPECIES"] 
    details = parameters["DETAILS"]
    subset = parameters["SUBSET"]
    hide = int(parameters["HIDE"]) 
    theta = float(parameters["THETA"])
    zeta = float(parameters["ZETA"])
    tscore_type = parameters["TSCORE_TYPE"]
    beta = int(parameters["BETA"])  
    pred = parameters["PRED"]
    dist = int(parameters["DIST"])
    type_param = parameters["TYPE"]
    exp_prefix = parameters["EXP_PREFIX"]
    grouping_type = parameters["GROUPING"] 
    supp_type = parameters["SUPP_TYPE"]
    drug_name = parameters["DRUG_NAME"]
    m_type = parameters["TYPE"]     
    
    input_dir = os.path.abspath("../../../data_input/"+species+"_res/")+"/"
    output_dir = os.path.abspath("../../../data_output/"+species+"_out/")+"/"
    format_dir = os.path.abspath("../../../data_output/"+species+"_format/")+"/"    
    
    input_fh = open(input_dir + "strains-all.txt")
    strains = readStrains(input_fh)
    input_fh.close()
    
    diffTime("Input reading...")
    
    input_fh = open(input_dir + "pred_res_profiles.txt")
    res_profiles = readResistanceProfiles(input_fh, strains, map=True)
    input_fh.close()
    
    cluster_fh = open(input_dir + "conn_comp-det-"+str(0)+".txt")
    clusters, id_cluster_map = readClusters(cluster_fh, strains)
    cluster_fh.close()
    
    strains_ord_fh = open(input_dir + "names-strains-ord.txt")
    strains_ord = readStrainNamesOrd(strains_ord_fh)
    strains_ord_fh.close()
    
    if os.path.exists(input_dir + "cc_mutability.txt"):
        mutability_fh = open(input_dir + "cc_mutability.txt")
        mutabilities = readMutabilities(mutability_fh, clusters, id_cluster_map, strains_ord)
        mutability_fh.close()
    else:
        mutabilities = None
    
    if os.path.exists(input_dir + "gene_mapped_functions.txt"):
        input_fh = open(input_dir + "gene_mapped_functions.txt")
        gene_functions = loadGeneFunctions(input_fh)
        input_fh.close()
    else:
        gene_functions = None

    diffTime(drug_name)
    print(res_profiles)
    drug_res_profile = copy.deepcopy(res_profiles[drug_name])
    drug_name = drug_res_profile.drug_name

    input_res_fn = output_dir +"/det/"+str(exp_prefix)+"-"+str(hide)+"-"+str(dist)+"-"+grouping_type+"-"+details+"-"+subset+"-"+m_type+"-"+pred+"-"+str(beta)+"-"+str(theta)+"-"+str(zeta)+"-"+str(tscore_type)+"-"+drug_name+".txt"
    print("HERE", input_res_fn)
    if not os.path.exists(input_res_fn):
        print("HERE2")
        return None

    input_res_fh = open(input_res_fn)
    output_res_fn = format_dir +"/html/"+str(exp_prefix)+"-"+str(hide)+"-"+str(dist)+"-"+grouping_type+"-"+details+"-"+subset+"-"+m_type+"-"+pred+"-"+str(beta)+"-"+str(theta)+"-"+str(zeta)+"-"+str(tscore_type)+"-"+drug_name+".html"
    output_fh = open(output_res_fn, "w")            

    hidden_strains, gene_profiles, gene_targets, drug_res_genes = readFeaturesOne(drug_name, strains, input_res_fh)

    genes_to_show = filterTopGeneProfiles(gene_profiles, 500)
    printHtml(genes_to_show, drug_name, strains, output_fh, show_profiles=True, hidden_res_profile=drug_res_profile, drug_res_genes=drug_res_genes, drug_targets=gene_targets, gene_functions=gene_functions, type=type, mutabilities=mutabilities)
    input_res_fh.close()

    output_fh.close()
    return None



if __name__ == '__main__':            
    parameters = readParameters(sys.argv)
    if "SPECIES" in parameters and not parameters["SPECIES"] in ["mtb","sta"]:
        species = parameters["SPECIES"]
    else:
        species = "mtb80"
    #species = "mtb80"
        
    if "W" in parameters:
        WORKERS = int(parameters["W"])
    else:
        WORKERS = 8

    input_dir = os.path.abspath("../../../data_input/"+species+"_res/")+"/"
    output_dir = os.path.abspath("../../../data_output/"+species+"_out/")+"/"
    format_dir = os.path.abspath("../../../data_output/"+species+"_format/")+"/"
    ensure_dir(format_dir)
    ensure_dir(format_dir +"/html/")
    
    details_param = "S"
    hide_param = 0
    dist_params = ["0"]
    exp_prefix = "FINAL3"
    grouping_param = "G"
    subset_param = "N"
    pred_params = ["A","B"]
    theta_params = ["0.2","0.5","0.8"]
    type_params = ["G","S"]
    zeta_params = ["0.0"]
    tscore_params = ["P"]

    TASKS = []
    TASK = {}

    input_fh = open(input_dir + "res_profiles_ret.txt")
    drug_names = readDrugNames(input_fh)
    input_fh.close()
    print(drug_names)

    for theta_param in theta_params:
        for tscore_param in  tscore_params:
            for zeta_param in zeta_params:
                for dist_param in dist_params:
                    for type_param in type_params:
                        if type_param == "G" and dist_param == "20":
                            continue
                        for pred_param in pred_params:
                            for beta_param in range(-1, 0, 1):
                                for supp_type_param in ["wsupport", "support", "or", "tscore", "gene_tscore"]:
                                    if supp_type_param != "tscore" and theta_param != "0.5":
                                        continue  
                                    for drug_name in drug_names:
                                        TASK["SPECIES"] = species
                                        TASK["PHASE"] = str(0)
                                        TASK["DETAILS"] = str(details_param)
                                        TASK["HIDE"] = str(hide_param)
                                        TASK["DIST"] = str(dist_param)
                                        TASK["GROUPING"] = str(grouping_param)
                                        TASK["EXP_PREFIX"] = exp_prefix 
                                        TASK["TYPE"] = type_param
                                        TASK["PRED"] = pred_param
                                        TASK["BETA"] = beta_param
                                        TASK["THETA"] = theta_param  
                                        TASK["ZETA"] = zeta_param  
                                        TASK["TSCORE_TYPE"] = tscore_param
                                        TASK["SUPP_TYPE"] = supp_type_param
                                        TASK["SUBSET"] = subset_param
                                        TASK["DRUG_NAME"] = drug_name
                                        
                                        TASKS.append(TASK.copy())
                            
    for TASK in TASKS:
        print(TASK)
    if WORKERS > 1:              
        pool = Pool(processes=WORKERS)   
        for r in pool.imap(decorateScors, TASKS):
            print(r)
    else:
        for T in TASKS:
            decorateScors(T)
        
    print("END")
    

